@@ -1,6 +1,7 @@
-from WebKit.Examples.ExamplePage import ExamplePage
 import os
 from glob import glob
+from WebKit.Examples.ExamplePage import ExamplePage
+
 
 class KidExamplePage(ExamplePage):
 
@@ -29,13 +30,13 @@ class KidExamplePage(ExamplePage):
 
 	def writeOtherMenu(self):
 		self.menuHeading('Other')
-		viewPath = self.request().uriWebKitRoot() + "KidKitExamples/View"
+		viewPath = self.request().uriWebKitRoot() + "KidKit/Examples/View"
 		self.menuItem('View source of<br/>%s' % self.title(),
 			self.request().uriWebKitRoot() +
-				'KidKitExamples/View?filename=%s'
+				'KidKit/Examples/View?filename=%s'
 				% os.path.basename(self.request().serverSidePath()))
 		if self.application().hasContext('Documentation'):
 			filename = 'Documentation/WebKit.html'
 			if os.path.exists(filename):
 				self.menuItem('Local WebKit docs',
-					self.request().adapterName() + '/' + filename)
+					self.request().servletPath() + '/' + filename)
