@@ -1,14 +1,14 @@
 name = 'KidKit'
 
-version = (0, 9, 2)
+version = (0, 9, 3)
 
-docs = [ {'name': "User's Guide", 'file': 'UsersGuide.html'} ]
+docs = [{'name': "User's Guide", 'file': 'UsersGuide.html'}]
 
-status = 'alpha'
+status = 'beta'
 
 requiredPyVersion = (2, 3, 0)
 
-requiredSoftware = [ { 'name': 'kid', 'version': (0, 6, 0) } ]
+requiredSoftware = [{'name': 'kid', 'version': (0, 6, 0)}]
 
 synopsis = """KidKit is a Webware plug-in that allows Kid templates
 to be automatically compiled and run as servlets by the WebKit application server.
@@ -25,9 +25,8 @@ WebKitConfig = {
 def willRunFunc():
 	# WebKit doesn't check requiredSoftware yet. So we do so:
 	try:
-		import imp
 		for soft in requiredSoftware:
-			imp.find_module(soft['name'])
+			__import__(soft['name'])
 	except ImportError:
 		success = 0
 	else:
