@@ -1,19 +1,115 @@
 #!/usr/bin/env python
 
-from UserDict import UserDict
+"""The Application singleton.
+
+`Application` and `AppServer` work together to setup up and dispatch requests.
+The distinction between the two is largely historical, but AppServer
+communicates directly with Adapters (or speaks protocols like HTTP), and
+Application receives the (slightly) processed input from AppServer and turns
+it into `Transaction`, `HTTPRequest`, `HTTPResponse`, and `Session`.
+
+Application is a singleton, which belongs to the AppServer. You can get access
+through the Transaction object (``transaction.application()``), or you can do::
+
+	from AppServer import globalAppServer
+	application = globalAppServer.application()
+
+Settings for Application are taken from ``Configs/Application.config``,
+and it is used for many global settings, even if they aren't closely tied
+to the Application object itself.
+
+"""
+
 from types import FloatType, ClassType
+from UserDict import UserDict
 
 from Common import *
 from Object import Object
 from ConfigurableForServerSidePath import ConfigurableForServerSidePath
 from ExceptionHandler import ExceptionHandler
 from HTTPRequest import HTTPRequest
+from HTTPExceptions import HTTPException, HTTPSessionExpired
 from Transaction import Transaction
-from Session import Session
+from ASStreamOut import ConnectionAbortedError
 import URLParser
-import HTTPExceptions
 
-debug = 0
+debug = False
+
+defaultConfig = {
+	'PrintConfigAtStartUp': True,
+	'LogActivity': True,
+	'ActivityLogFilename': 'Logs/Activity.csv',
+	'ActivityLogColumns': [
+		'request.remoteAddress', 'request.method',
+		'request.uri', 'response.size',
+		'servlet.name', 'request.timeStamp',
+		'transaction.duration',
+		'transaction.errorOccurred'
+		],
+	'SessionModule': 'Session',
+	'SessionStore': 'Dynamic',
+	'SessionTimeout': 60,
+	'SessionPrefix': '',
+	'SessionName': '_SID_',
+	'IgnoreInvalidSession': True,
+	'UseAutomaticPathSessions': False,
+	'ShowDebugInfoOnErrors': True,
+	'IncludeFancyTraceback': False,
+	'FancyTracebackContext': 5,
+	'UserErrorMessage': 'The site is having technical difficulties'
+		' with this page. An error has been logged, and the problem'
+		' will be fixed as soon as possible. Sorry!',
+	'LogErrors': True,
+	'ErrorLogFilename': 'Logs/Errors.csv',
+	'SaveErrorMessages': True,
+	'ErrorMessagesDir': 'ErrorMsgs',
+	'EmailErrors': False,
+	'EmailErrorReportAsAttachment': False,
+	'ErrorEmailServer': 'localhost',
+	'ErrorEmailHeaders': {
+		'From': 'webware@mydomain',
+		'To': ['webware@mydomain'],
+		'Reply-to': 'webware@mydomain',
+		'content-type': 'text/html',
+		'Subject': 'Error'
+		},
+	'ErrorPage': None,
+	'MaxValueLengthInExceptionReport': 500,
+	'RPCExceptionReturn': 'traceback',
+	'ReportRPCExceptionsInWebKit': True,
+	'Contexts': {
+		'default': 'Examples',
+		'Admin': 'Admin',
+		'Examples': 'Examples',
+		'Testing': 'Testing',
+		'Docs': 'Docs',
+		},
+	'Debug': {
+		'Sessions': False,
+		},
+	'EnterDebuggerOnException': False,
+	'DirectoryFile': ['index', 'Index', 'main', 'Main'],
+	'UseCascadingExtensions': True,
+	'ExtensionCascadeOrder': ['.py', '.psp', '.kid', '.html'],
+	'ExtraPathInfo': True,
+	'ExtensionsToIgnore': [
+		'.pyc', '.pyo', '.tmpl', '.bak', '.py_bak',
+		'.py~', '.psp~', '.kid~', '.html~', '.tmpl~'
+		],
+	'ExtensionsToServe': [],
+	'FilesToHide': [
+		'.*', '*~', '*.bak', '*.py_bak', '*.tmpl',
+		'*.pyc', '*.pyo', '__init__.*', '*.config'
+		],
+	'FilesToServe': [],
+	'UnknownFileTypes': {
+		'ReuseServlets': True,
+		'Technique': 'serveContent', # or redirectSansAdapter
+		'CacheContent': False,
+		'MaxCacheContentSize': 128*1024,
+		'ReadBufferSize': 32*1024
+		},
+}
 
 
 class EndResponse(Exception):
@@ -32,30 +128,14 @@ class EndResponse(Exception):
 class Application(ConfigurableForServerSidePath, Object):
 	"""The Application singleton.
 
-	`Application` and `AppServer` work together to setup up and dispatch
-	requests. The distinction between the two is largely historical, but
-	AppServer communicates directly with Adapters (or speaks protocols like
-	HTTP), and Application receives the (slightly) processed input from
-	AppServer and turns it into `Transaction`, `HTTPRequest`, `HTTPResponse`,
-	and `Session`.
-
-	Application is a singleton, which belongs to the AppServer. You can get
-	access through the Transaction object (``transaction.application()``),
-	or you can do::
-
-	    from AppServer import globalAppServer
-	    application = globalAppServer.application()
-
-	Settings for Application are taken from ``Configs/Application.config``,
-	and it is used for many global settings, even if they aren't closely tied
-	to the Application object itself.
+	Purpose and usage are explained in the module docstring.
 
 	"""
 
 
 	## Init ##
 
-	def __init__(self, server):
+	def __init__(self, server, useSessionSweeper=1):
 		"""Called only by `AppServer`, sets up the Application."""
 
 		self._server = server
@@ -66,10 +146,14 @@ class Application(ConfigurableForServerSidePath, Object):
 		ConfigurableForServerSidePath.__init__(self)
 		Object.__init__(self)
 
+		print 'Initializing Application...'
+		print 'Current directory:', os.getcwd()
+
 		if self.setting('PrintConfigAtStartUp'):
 			self.printConfig()
 
 		self.initVersions()
+		self.initErrorPage()
 
 		self._shutDownHandlers = []
 
@@ -86,41 +170,89 @@ class Application(ConfigurableForServerSidePath, Object):
 		# sessions, in case the loading of the sessions causes an exception.
 		self._exceptionHandlerClass = ExceptionHandler
 
-		self._session_prefix = self.setting('SessionPrefix', None) or ''
-		if self._session_prefix:
-			if self._session_prefix == 'hostname':
-				from MiscUtils.Funcs import hostName
-				self._session_prefix = hostName()
-			self._session_prefix += '-'
-		self._session_timeout = self.setting('SessionTimeout')*60
-		self._session_name = self.setting('SessionName', None) or '_SID_'
-
-		# For session store:
-		session_store = 'Session%sStore' % self.setting('SessionStore')
-		exec 'from %s import %s' % (session_store, session_store)
-		klass = locals()[session_store]
-		assert isinstance(klass, ClassType) or issubclass(klass, Object)
-		self._sessions = klass(self)
-
-		print 'Current directory:', os.getcwd()
+		self.initSessions()
 
 		URLParser.initApp(self)
 		self._rootURLParser = URLParser.ContextParser(self)
 
-		self.running = 1
+		self._running = 1
 
-		self.startSessionSweeper()
+		if useSessionSweeper:
+			self.startSessionSweeper()
 
-		try: # try to get a 404 error page from the working dir
-			self._404Page = open(os.path.join(self._serverSidePath,
-				"404Text.txt")).read()
-		except: # if not found in the working dir,
-			try: # then try the directory this file is located in
-				self._404Page = open(os.path.join(
-					os.path.dirname(os.path.abspath(__file__)),
-					"404Text.txt")).read()
-			except: # otherwise fall back to standard exception
-				self._404Page = None
+	def initErrorPage(self):
+		"""Initialize the error page related attributes."""
+		dirs = (self._serverSidePath,
+			os.path.dirname(os.path.abspath(__file__)))
+		pages = ('error404.html', '404Text.txt')
+		for dir in dirs:
+			for page in pages:
+				try:
+					self._error404 = open(os.path.join(dir, page)).read()
+					if page != pages[0]:
+						print 'Deprecation warning: ' \
+							'Please use %s instead of %s' % pages
+				except Exception:
+					continue
+				else:
+					break
+			else:
+				continue
+			break
+		else:
+			self._error404 = None
+		urls = self.setting('ErrorPage') or None
+		if urls:
+			try:
+				errors = urls.keys()
+			except AttributeError:
+				errors = ['Exception']
+				urls = {errors[0]: urls}
+			for err in errors:
+				if urls[err] and not urls[err].startswith('/'):
+					urls[err] = '/' + urls[err]
+		self._errorPage = urls
+
+	def initSessions(self):
+		"""Initialize all session related attributes."""
+		self._sessionPrefix = self.setting('SessionPrefix') or ''
+		if self._sessionPrefix:
+			if self._sessionPrefix == 'hostname':
+				from MiscUtils.Funcs import hostName
+				self._sessionPrefix = hostName()
+			self._sessionPrefix += '-'
+		self._sessionTimeout = self.setting('SessionTimeout')*60
+		self._sessionName = self.setting('SessionName') \
+			or self.defaultConfig()['SessionName']
+		self._autoPathSessions = self.setting('UseAutomaticPathSessions')
+		moduleName = self.setting('SessionModule')
+		className = moduleName.split('.')[-1]
+		try:
+			exec 'from %s import %s' % (moduleName, className)
+			klass = locals()[className]
+			if not isinstance(klass, ClassType) \
+					and not issubclass(klass, Object):
+				raise ImportError
+			self._sessionClass = klass
+		except ImportError:
+			print "ERROR: Could not import Session class '%s'" \
+				" from module '%s'" % (className, moduleName)
+			self._sessionClass = None
+		moduleName = self.setting('SessionStore')
+		if moduleName in ('Dynamic', 'File', 'Memory'):
+			moduleName = 'Session%sStore' % moduleName
+		className = moduleName.split('.')[-1]
+		try:
+			exec 'from %s import %s' % (moduleName, className)
+			klass = locals()[className]
+			if not isinstance(klass, ClassType) \
+					and not issubclass(klass, Object):
+				raise ImportError
+			self._sessions = klass(self)
+		except ImportError:
+			print "ERROR: Could not import SessionStore class '%s'" \
+				" from module '%s'" % (className, moduleName)
+			self._sessions = None
 
 	def initVersions(self):
 		"""Get and store versions.
@@ -150,13 +282,13 @@ class Application(ConfigurableForServerSidePath, Object):
 		session objects (and disk copies of those objects) that have expired.
 
 		"""
-		if self._session_timeout:
+		if self._sessionTimeout:
 			tm = self.taskManager()
 			if tm:
 				from Tasks import SessionTask
 				import time
 				task = SessionTask.SessionTask(self._sessions)
-				sweepinterval = self._session_timeout/10
+				sweepinterval = self._sessionTimeout/10
 				tm.addPeriodicAction(time.time() + sweepinterval,
 					sweepinterval, task, "SessionSweeper")
 				print "Session sweeper has started."
@@ -169,7 +301,7 @@ class Application(ConfigurableForServerSidePath, Object):
 
 		"""
 		print "Application is shutting down..."
-		self.running = 0
+		self._running = 0
 		self._sessions.storeAllSessions()
 		if self._server.isPersistent():
 			self.taskManager().stop()
@@ -198,76 +330,7 @@ class Application(ConfigurableForServerSidePath, Object):
 
 	def defaultConfig(self):
 		"""The default Application.config."""
-		return {
-			'PrintConfigAtStartUp': 1,
-			'LogActivity': 1,
-			'ActivityLogFilename': 'Logs/Activity.csv',
-			'ActivityLogColumns': [
-				'request.remoteAddress', 'request.method',
-				'request.uri', 'response.size',
-				'servlet.name', 'request.timeStamp',
-				'transaction.duration',
-				'transaction.errorOccurred'
-				],
-			'SessionStore': 'Dynamic',
-			'SessionTimeout': 60,
-			'IgnoreInvalidSession': 1,
-			'UseAutomaticPathSessions': 0,
-			'ShowDebugInfoOnErrors': 1,
-			'IncludeFancyTraceback': 0,
-			'FancyTracebackContext': 5,
-			'UserErrorMessage': 'The site is having technical difficulties'
-				' with this page. An error has been logged, and the problem'
-				' will be fixed as soon as possible. Sorry!',
-			'LogErrors': 1,
-			'ErrorLogFilename': 'Logs/Errors.csv',
-			'SaveErrorMessages': 1,
-			'ErrorMessagesDir': 'ErrorMsgs',
-			'EmailErrors': 0,
-			'ErrorEmailServer': 'localhost',
-			'ErrorEmailHeaders': {
-				'From': 'webware@mydomain',
-				'To': ['webware@mydomain'],
-				'Reply-to': 'webware@mydomain',
-				'content-type': 'text/html',
-				'Subject': 'Error'
-				},
-			'MaxValueLengthInExceptionReport': 500,
-			'RPCExceptionReturn': 'traceback',
-			'ReportRPCExceptionsInWebKit': 1,
-			'Contexts': {
-				'default': 'Examples',
-				'Admin': 'Admin',
-				'Examples': 'Examples',
-				'Testing': 'Testing',
-				'Docs': 'Docs',
-				},
-			'Debug': {
-				'Sessions': 0,
-				},
-			'EnterDebuggerOnException': 0,
-			'DirectoryFile': ['index', 'Index', 'main', 'Main'],
-			'UseCascadingExtensions': 1,
-			'ExtensionCascadeOrder': ['.py','.psp','.kid','.html'],
-			'ExtraPathInfo': 1,
-			'ExtensionsToIgnore': [
-				'.pyc', '.pyo', '.tmpl', '.bak', '.py_bak',
-				'.py~', '.psp~', '.kid~', '.html~', '.tmpl~'
-				],
-			'ExtensionsToServe': [],
-			'FilesToHide': [
-				'.*', '*~', '*.bak', '*.py_bak', '*.tmpl',
-				'*.pyc', '*.pyo', '__init__.*', '*.config'
-				],
-			'FilesToServe': [],
-			'UnknownFileTypes': {
-				'ReuseServlets': 1,
-				'Technique': 'serveContent', # serveContent or redirectSansAdapter
-				'CacheContent': 0,
-				'MaxCacheContentSize': 128*1024,
-				'ReadBufferSize': 32*1024
-				},
-		}
+		return defaultConfig # defined on the module level
 
 	def configFilename(self):
 		return self.serverSidePath('Configs/Application.config')
@@ -278,18 +341,6 @@ class Application(ConfigurableForServerSidePath, Object):
 
 	## Versions ##
 
-	def version(self):
-		"""Return the version of the application.
-
-		This implementation returns '0.1'. Subclasses should
-		override to return the correct version number.
-
-		"""
-		# @@ 2000-05-01 ce: Maybe this could be a setting 'AppVersion'
-		# @@ 2003-03 ib: Does anyone care about this?  What's this
-		# version even supposed to mean?
-		return '0.1'
-
 	def webwareVersion(self):
 		"""Return the Webware version as a tuple."""
 		return self._webwareVersion
@@ -330,7 +381,7 @@ class Application(ConfigurableForServerSidePath, Object):
 		"""A dictionary of all the session objects."""
 		return self._sessions
 
-	def createSessionForTransaction(self, transaction):
+	def createSessionForTransaction(self, trans):
 		"""Get the session object for the transaction.
 
 		If the session already exists, returns that, otherwise creates
@@ -342,7 +393,7 @@ class Application(ConfigurableForServerSidePath, Object):
 		debug = self.setting('Debug').get('Sessions')
 		if debug:
 			prefix = '>> [session] createSessionForTransaction:'
-		sessId = transaction.request().sessionId()
+		sessId = trans.request().sessionId()
 		if debug:
 			print prefix, 'sessId =', sessId
 		if sessId:
@@ -351,28 +402,63 @@ class Application(ConfigurableForServerSidePath, Object):
 				if debug:
 					print prefix, 'retrieved session =', session
 			except KeyError:
-				transaction.request().setSessionExpired(1)
+				trans.request().setSessionExpired(1)
 				if not self.setting('IgnoreInvalidSession'):
-					raise HTTPExceptions.HTTPSessionExpired
+					raise HTTPSessionExpired
 				sessId = None
 		if not sessId:
-			session = Session(transaction)
+			session = self._sessionClass(trans)
 			self._sessions[session.identifier()] = session
 			if debug:
 				print prefix, 'created session =', session
-		transaction.setSession(session)
+		trans.setSession(session)
 		return session
 
-	def createSessionWithID(self, transaction, sessionID):
-		# Create a session object with our session ID
-		sess = Session(transaction, sessionID)
+	def createSessionWithID(self, trans, sessionID):
+		"""Create a session object with our session ID."""
+		sess = self._sessionClass(trans, sessionID)
 		# Replace the session if it didn't already exist,
 		# otherwise we just throw it away.  setdefault is an atomic
 		# operation so this guarantees that 2 different
 		# copies of the session with the same ID never get placed into
 		# the session store, even if multiple threads are calling
 		# this method simultaneously.
-		transaction.application()._sessions.setdefault(sessionID, sess)
+		trans.application()._sessions.setdefault(sessionID, sess)
+
+	def sessionTimeout(self, trans):
+		"""Get the timeout (in seconds) for a user session.
+
+		Overwrite to make this transaction dependent.
+
+		"""
+		return self._sessionTimeout
+
+	def sessionPrefix(self, trans):
+		"""Get the prefix string for the session ID.
+
+
+		Overwrite to make this transaction dependent.
+
+		"""
+		return self._sessionPrefix
+
+	def sessionName(self, trans):
+		"""Get the name of the field holding the session ID.
+
+
+		Overwrite to make this transaction dependent.
+
+		"""
+		return self._sessionName
+
+	def sessionCookiePath(self, trans):
+		"""Get the cookie path for this transaction.
+
+		The servlet path is used for security reasons, see:
+		http://www.net-security.org/dl/articles/cookie_path.pdf
+
+		"""
+		return trans.request().servletPath() + '/'
 
 
 	## Misc Access ##
@@ -411,7 +497,7 @@ class Application(ConfigurableForServerSidePath, Object):
 
 	## Activity Log ##
 
-	def writeActivityLog(self, transaction):
+	def writeActivityLog(self, trans):
 		"""Write an entry to the activity log.
 
 		Writes an entry to the script log file. Uses settings
@@ -430,16 +516,16 @@ class Application(ConfigurableForServerSidePath, Object):
 		# NamedValueAccess and reponds to valueForName()
 		objects = UserDict({
 			'application': self,
-			'transaction': transaction,
-			'request': transaction.request(),
-			'response': transaction.response(),
-			'servlet': transaction.servlet(),
-			'session': transaction._session, # don't cause creation of session
+			'transaction': trans,
+			'request': trans.request(),
+			'response': trans.response(),
+			'servlet': trans.servlet(),
+			'session': trans._session, # don't cause creation of session
 		})
 		for column in self.setting('ActivityLogColumns'):
 			try:
 				value = objects.valueForName(column)
-			except:
+			except Exception:
 				value = '(unknown)'
 			if type(value) is FloatType:
 				# probably need more flexibility in the future
@@ -467,28 +553,35 @@ class Application(ConfigurableForServerSidePath, Object):
 		exceptions, which are then passed on to `handleExceptionInTransaction`.
 
 		"""
-		trans = None
-		try:
-			request = self.createRequestForDict(requestDict)
+		request = self.createRequestForDict(requestDict)
+		if request:
 			trans = Transaction(application=self, request=request)
-			request.setTransaction(trans)
-			response = request.responseClass()(trans, strmOut)
-			trans.setResponse(response)
-			self.runTransaction(trans)
-			trans.response().deliver()
-		except:
 			if trans:
-				trans.setErrorOccurred(1)
-			if self.setting('EnterDebuggerOnException') and sys.stdin.isatty():
-				import pdb
-				pdb.post_mortem(sys.exc_info()[2])
-			self.handleExceptionInTransaction(sys.exc_info(), trans)
-			trans.response().deliver()
-
-		if self.setting('LogActivity'):
-			self.writeActivityLog(trans)
-		request.clearTransaction()
-		response.clearTransaction()
+				request.setTransaction(trans)
+				response = request.responseClass()(trans, strmOut)
+				if response:
+					trans.setResponse(response)
+					self.runTransaction(trans)
+					try:
+						trans.response().deliver()
+					except ConnectionAbortedError, err:
+						trans.setError(err)
+					response.clearTransaction()
+					# release possible servlets on the stack
+					while 1:
+						servlet = request.pop()
+						if not servlet:
+							break
+						self.returnServlet(servlet)
+						servlet.resetKeyBindings()
+					# get current servlet (this may have changed)
+					servlet = trans.servlet()
+					if servlet:
+						# return the current servlet to its pool
+						self.returnServlet(servlet)
+				if self.setting('LogActivity'):
+					self.writeActivityLog(trans)
+			request.clearTransaction()
 		return trans
 
 	def createRequestForDict(self, requestDict):
@@ -522,30 +615,69 @@ class Application(ConfigurableForServerSidePath, Object):
 		Called by `dispatchRawRequest`.
 
 		"""
-		# @@ gtalvola: I'm guessing this is not the ideal place
-		# @@ to put this code. But, it works.
-		if self.setting('UseAutomaticPathSessions'):
-			request = trans.request()
-			request_has_cookie_session = request.hasCookieSession()
-			request_has_path_session = request.hasPathSession()
-			if request_has_cookie_session and request_has_path_session:
-				self.handleUnnecessaryPathSession(trans)
-				return
-			elif not request_has_cookie_session and not request_has_path_session:
-				self.handleMissingPathSession(trans)
-				return
-		servlet = None
+		findServlet = self.rootURLParser().findServletForTransaction
 		try:
-			servlet = self._rootURLParser.findServletForTransaction(trans)
+			# remove the session identifier from the path
+			self.removePathSession(trans)
+			# determine the context and the servlet for the transaction
+			servlet = findServlet(trans)
+			# handle session field only now, because the name of the
+			# session id field can depend on the context
+			self.handlePathSession(trans)
+			# now everything is set, run the transaction
 			self.runTransactionViaServlet(servlet, trans)
-		except HTTPExceptions.HTTPException, err:
-			err.setTransaction(trans)
-			trans.response().displayError(err)
 		except EndResponse:
 			pass
-		if servlet:
-			# Return the servlet to its pool
-			self.returnServlet(servlet, trans)
+		except ConnectionAbortedError, err:
+			trans.setError(err)
+		except Exception, err:
+			urls = {}
+			while 1:
+				trans.setError(err)
+				isHTTPException = isinstance(err, HTTPException)
+				if isHTTPException:
+					err.setTransaction(trans)
+				# Get custom error page corresponding to the exception,
+				# but do not use custom page if response is already committed:
+				if not self._errorPage or trans.response().isCommitted():
+					break
+				url = self.errorPage(err.__class__)
+				if isHTTPException and not url:
+					# get custom error page for status code
+					code = err.code()
+					if self._errorPage.has_key(code):
+						url = self._errorPage[code]
+				if not url or urls.has_key(url):
+					# If there is no custom error page configured,
+					# or we get into a circular chain of error pages,
+					# then we fall back to standard error handling.
+					break
+				urls[url] = None
+				# forward to custom error page
+				try:
+					self.forward(trans, url)
+				except EndResponse:
+					pass
+				except ConnectionAbortedError, err:
+					trans.setError(err)
+				except Exception, err:
+					# If the custom error page itself throws an exception,
+					# display the new exception instead of the original one,
+					# so we notice that something is broken here.
+					url = None
+				if url:
+					return # error has already been handled
+			if isHTTPException:
+				# display standard http error page
+				trans.response().displayError(err)
+			else:
+				# standard error handling
+				if self.setting('EnterDebuggerOnException') \
+						and sys.stdin.isatty():
+					import pdb
+					pdb.post_mortem(sys.exc_info()[2])
+				self.handleExceptionInTransaction(
+					sys.exc_info(), trans)
 
 	def runTransactionViaServlet(self, servlet, trans):
 		"""Execute the transaction using the servlet.
@@ -584,16 +716,11 @@ class Application(ConfigurableForServerSidePath, Object):
 		You can change the request in place to control the servlet you are
 		forwarding to -- using methods like `HTTPRequest.setField`.
 
-		@@ 2003-03 ib: how does the forwarded servlet knows that it's not
-		the original servlet?
-
 		"""
 		# Reset the response to a "blank slate"
 		trans.response().reset()
-
 		# Include the other servlet
 		self.includeURL(trans, url)
-
 		# Raise an exception to end processing of this request
 		raise EndResponse
 
@@ -610,37 +737,27 @@ class Application(ConfigurableForServerSidePath, Object):
 		and ``**kw`` are passed as arguments to that method).
 
 		"""
-		urlPath = self.resolveInternalRelativePath(trans, url)
-		req = trans.request()
-		currentPath = req.urlPath()
-		currentServlet = trans._servlet
-		currentServerSidePath = req._serverSidePath
-		currentServerSideContextPath = req._serverSideContextPath
-		currentContextName = req._contextName
-		currentServerRootPath = req._serverRootPath
-		currentExtraURLPath = req._extraURLPath
-		req.setURLPath(urlPath)
-		req.addParent(currentServlet)
-
-		servlet = self._rootURLParser.findServletForTransaction(trans)
-		trans._servlet = servlet
+		# store current request and set the new URL
+		request = trans.request()
+		request.push(trans.servlet(),
+			self.resolveInternalRelativePath(trans, url))
+		# get new servlet
+		servlet = self.rootURLParser().findServletForTransaction(trans)
+		trans.setServlet(servlet)
+		# call method of included servlet
 		if hasattr(servlet, 'runMethodForTransaction'):
-			result = servlet.runMethodForTransaction(trans, method, *args, **kw)
+			result = servlet.runMethodForTransaction(
+				trans, method, *args, **kw)
 		else:
 			servlet.awake(trans)
 			result = getattr(servlet, method)(*args, **kw)
 			servlet.sleep(trans)
-
-		# Put things back
-		req.setURLPath(currentPath)
-		req._serverSidePath = currentServerSidePath
-		req._serverSideContextPath = currentServerSideContextPath
-		req._contextName = currentContextName
-		req._serverRootPath = currentServerRootPath
-		req._extraURLPath = currentExtraURLPath
-		req.popParent()
-		trans._servlet = currentServlet
-
+		# return new servlet to its pool
+		self.returnServlet(servlet)
+		# release bindings of new servlet
+		servlet.resetKeyBindings()
+		# restore current request
+		trans.setServlet(request.pop())
 		return result
 
 	def includeURL(self, trans, url):
@@ -650,36 +767,26 @@ class Application(ConfigurableForServerSidePath, Object):
 		except control is ultimately returned to the servlet.
 
 		"""
-		urlPath = self.resolveInternalRelativePath(trans, url)
-		req = trans.request()
-		currentPath = req.urlPath()
-		currentServlet = trans._servlet
-		currentServerSidePath = req._serverSidePath
-		currentServerSideContextPath = req._serverSideContextPath
-		currentContextName = req._contextName
-		req.setURLPath(urlPath)
-		req.addParent(currentServlet)
-
-		# Run the included servlet.
-		# (2006-07-05 cz: Do not use try/finally here, because exception
-		# handling should happen in the context of the included servlet.)
-		servlet = self._rootURLParser.findServletForTransaction(trans)
-		trans._servlet = servlet
-		# We will interpret an EndResponse in an included page to mean that
-		# the current page may continue processing.
+		# store current request and set the new URL
+		request = trans.request()
+		request.push(trans.servlet(),
+			self.resolveInternalRelativePath(trans, url))
+		# get new servlet
+		servlet = self.rootURLParser().findServletForTransaction(trans)
+		trans.setServlet(servlet)
+		# run the included servlet
 		try:
 			servlet.runTransaction(trans)
 		except EndResponse:
+			# we interpret an EndResponse in an included page to mean
+			# that the current page may continue processing
 			pass
-		self.returnServlet(servlet, trans)
-
-		# Restore everything properly
-		req.popParent()
-		req.setURLPath(currentPath)
-		req._serverSidePath = currentServerSidePath
-		req._serverSideContextPath = currentServerSideContextPath
-		req._contextName = currentContextName
-		trans._servlet = currentServlet
+		# return new servlet to its pool
+		self.returnServlet(servlet)
+		# release bindings of new servlet
+		servlet.resetKeyBindings()
+		# restore current request
+		trans.setServlet(request.pop())
 
 	def resolveInternalRelativePath(self, trans, url):
 		"""Return the absolute internal path.
@@ -696,7 +803,7 @@ class Application(ConfigurableForServerSidePath, Object):
 				if not origDir.endswith('/'):
 					origDir += '/'
 			url = origDir + url
-		# Deal with . and .. in the path:
+		# deal with . and .. in the path
 		parts = []
 		for part in url.split('/'):
 			if parts and part == '..':
@@ -705,8 +812,19 @@ class Application(ConfigurableForServerSidePath, Object):
 				parts.append(part)
 		return '/'.join(parts)
 
-	def returnServlet(self, servlet, trans):
-		servlet.close(trans)
+	def returnServlet(self, servlet):
+		"""Return the servlet to its pool."""
+		servlet.close()
+
+	def errorPage(self, errorClass):
+		"""Get the error page url corresponding to an error class."""
+		if self._errorPage.has_key(errorClass.__name__):
+			return self._errorPage[errorClass.__name__]
+		if errorClass is not Exception:
+			for errorClass in errorClass.__bases__:
+				url = self.errorPage(errorClass)
+				if url:
+					return url
 
 	def handleException(self):
 		"""Handle exceptions.
@@ -718,7 +836,7 @@ class Application(ConfigurableForServerSidePath, Object):
 		"""
 		self._exceptionHandlerClass(self, None, sys.exc_info())
 
-	def handleExceptionInTransaction(self, excInfo, transaction):
+	def handleExceptionInTransaction(self, excInfo, trans):
 		"""Handle exception with info.
 
 		Handles exception `excInfo` (as returned by ``sys.exc_info()``)
@@ -727,10 +845,10 @@ class Application(ConfigurableForServerSidePath, Object):
 		`ExceptionHandler.ExceptionHandler`.
 
 		"""
-		req = transaction.request()
-		editlink = req.adapterName() + "/Admin/EditFile"
-		self._exceptionHandlerClass(self, transaction,
-			excInfo, {"editlink": editlink})
+		request = trans.request()
+		editlink = request.adapterName() + "/Admin/EditFile"
+		self._exceptionHandlerClass(self, trans, excInfo,
+			{"editlink": editlink})
 
 	def rootURLParser(self):
 		"""Accessor: the Rool URL parser.
@@ -773,7 +891,53 @@ class Application(ConfigurableForServerSidePath, Object):
 		# @@ 2003-02 ib: does anyone care?
 		pass
 
-	def handleMissingPathSession(self, transaction):
+	def removePathSession(self, trans):
+		"""Remove a possible session identifier from the path."""
+		request = trans.request()
+		# Try to get automatic path session:
+		# If UseAutomaticPathSessions is enabled in Application.config,
+		# Application redirects the browser to a URL with SID in path:
+		# http://gandalf/a/_SID_=2001080221301877755/Examples/
+		# The _SID_ part is extracted and removed from path here.
+		# Note that We do not check for the exact name of the field
+		# here because it may be context dependent.
+		p = request._pathInfo
+		if p:
+			p = p.split('/', 2)
+			if len(p) > 1 and not p[0] and '=' in p[1]:
+				s = p[1]
+				request._pathSID = s.split('=', 1)
+				s += '/'
+				del p[1]
+				env = request.environ()
+				request._pathInfo = env['PATH_INFO'] = '/'.join(p)
+				for v in ('REQUEST_URI', 'PATH_TRANSLATED'):
+					if env.has_key(v):
+						env[v] = env[v].replace(s, '', 1)
+			else:
+				request._pathSID = None
+		else:
+			request._pathSID = None
+
+	def handlePathSession(self, trans):
+		"""Handle the session identifier that has been found in the path."""
+		request = trans.request()
+		if request._pathSID:
+			sessionName = self.sessionName(trans)
+			if request._pathSID[0] == sessionName:
+				if request.hasCookie(sessionName):
+					self.handleUnnecessaryPathSession(trans)
+				request.cookies()[sessionName] = request._pathSID[1]
+			else:
+				if self._autoPathSessions:
+					if not request.hasCookie(sessionName):
+						self.handleMissingPathSession(trans)
+		else:
+			if self._autoPathSessions:
+				if not request.hasCookie(self.sessionName(trans)):
+					self.handleMissingPathSession(trans)
+
+	def handleMissingPathSession(self, trans):
 		"""Redirect requests without session info in the path.
 
 		if UseAutomaticPathSessions is enabled in Application.config
@@ -785,17 +949,19 @@ class Application(ConfigurableForServerSidePath, Object):
 		depend on cookie support.
 
 		"""
-		newSid = transaction.session().identifier()
-		request = transaction.request()
-		url = '%s/%s=%s/%s%s%s' % (request.adapterName(), self._session_name,
-			newSid, request.pathInfo(), request.extraURLPath() or '',
+		newSid = trans.session().identifier()
+		request = trans.request()
+		url = '%s/%s=%s%s%s%s' % (request.servletPath(),
+			self.sessionName(trans), newSid,
+			request.pathInfo(), request.extraURLPath() or '',
 			request.queryString() and '?' + request.queryString() or '')
 		if self.setting('Debug')['Sessions']:
-			print '>> [sessions] handling UseAutomaticPathSessions,' \
-				' redirecting to', url
-		transaction.response().sendRedirect(url)
+			print '>> [sessions] handling UseAutomaticPathSessions, ' \
+				'redirecting to', url
+		trans.response().sendRedirect(url)
+		raise EndResponse
 
-	def handleUnnecessaryPathSession(self, transaction):
+	def handleUnnecessaryPathSession(self, trans):
 		"""Redirect request with unnecessary session info in the path.
 
 		This is called if it has been determined that the request has a path
@@ -803,48 +969,12 @@ class Application(ConfigurableForServerSidePath, Object):
 		unnecessary path session.
 
 		"""
-		request = transaction.request()
-		url = '%s/%s%s%s' % (request.adapterName(),
+		request = trans.request()
+		url = '%s%s%s%s' % (request.servletPath(),
 			request.pathInfo(), request.extraURLPath() or '',
 			request.queryString() and '?' + request.queryString() or '')
 		if self.setting('Debug')['Sessions']:
-			print ">> [sessions] handling unnecessary path session,' \
-				' redirecting to", url
-		transaction.response().sendRedirect(url)
-
-
-## Main ##
-
-def main(requestDict):
-	"""Return a raw reponse.
-
-	This method is mostly used by OneShotAdapter.py.
-
-	"""
-	from WebUtils.HTMLForException import HTMLForException
-	try:
-		assert type(requestDict) is type({})
-		app = Application(useSessionSweeper=0)
-		return app.runRawRequest(requestDict).response().rawResponse()
-	except:
-		return {
-			'headers': [('Content-type', 'text/html')],
-			'contents': '<html><body>%s</html></body>' % HTMLForException()
-		}
-
-
-"""
-You can run Application as a main script, in which case it expects a
-single argument which is a file containing a dictionary representing
-a request. This technique isn't very popular as Application itself
-could raise exceptions that aren't caught. See `CGIAdapter` and
-`AppServer` for a better example of how things should be done.
-
-Largely historical.
-"""
-
-if __name__ == '__main__':
-	if len(sys.argv) != 2:
-		sys.stderr.write('WebKit: Application: Expecting one filename argument.\n')
-	requestDict = eval(open(sys.argv[1]).read())
-	main(requestDict)
+			print ">> [sessions] handling unnecessary path session, ' \
+				'redirecting to", url
+		trans.response().sendRedirect(url)
+		raise EndResponse
