@@ -1,12 +1,10 @@
+from types import DictType, ListType
+import traceback, random, MimeWriter, smtplib
+
 from Common import *
-import time, traceback, types, random, sys, MimeWriter, smtplib, StringIO
-from time import asctime, localtime
 from MiscUtils.Funcs import dateForEmail
 from WebUtils.HTMLForException import HTMLForException
 from WebUtils.Funcs import htmlForDict, htmlEncode
-from HTTPResponse import HTTPResponse
-from types import DictType, StringType
-
 
 try: # linecache update workaround for Python < 2.4
 	traceback.format_exc # check version
@@ -17,7 +15,8 @@ else: # Python >= 2.4
 	def checkcache(): pass
 
 
-class singleton: pass
+class Singleton:
+	pass
 
 
 class ExceptionHandler(Object):
@@ -60,7 +59,7 @@ class ExceptionHandler(Object):
 
 		class ExceptionHandler(_ExceptionHandler):
 
-			hideValuesForFields = _ExceptionHandler.hideValuesForFields + ['foo', 'bar']
+			_hideValuesForFields = _ExceptionHandler._hideValuesForFields + ['foo', 'bar']
 
 			def work(self):
 				_ExceptionHandler.work(self)
@@ -75,18 +74,18 @@ class ExceptionHandler(Object):
 
 	"""
 	# keep these lower case to support case insensitivity:
-	hideValuesForFields = ['password', 'passwd', 'pwd',
-		'creditcard', 'credit card', 'cc']
+	_hideValuesForFields = ['password', 'passwd', 'pwd',
+		'creditcard', 'credit card', 'cc', 'pin', 'tan']
 	if 0: # for testing
-		hideValuesForFields.extend(['application', 'uri',
+		_hideValuesForFields.extend(['application', 'uri',
 			'http_accept', 'userid'])
-	hiddenString = '*** hidden ***'
+	_hiddenString = '*** hidden ***'
 
 
 	## Init ##
 
 	def __init__(self, application, transaction, excInfo,
-				 formatOptions=None):
+			formatOptions=None):
 		"""Create an exception handler instance.
 
 		ExceptionHandler instances are created anew for each exception.
@@ -120,8 +119,8 @@ class ExceptionHandler(Object):
 		# Making remaining code safe for no transaction.
 		#
 		# if self._res is None:
-		# 	self._res = HTTPResponse()
-		# 	self._tra.setResponse(self._res)
+		#      self._res = HTTPResponse()
+		#      self._tra.setResponse(self._res)
 
 		# Cache MaxValueLengthInExceptionReport for speed
 		self._maxValueLength = self.setting('MaxValueLengthInExceptionReport')
@@ -143,7 +142,7 @@ class ExceptionHandler(Object):
 		"""The full filesystem path for the servlet."""
 		try:
 			return self._tra.request().serverSidePath()
-		except:
+		except Exception:
 			return None
 
 	def basicServletName(self):
@@ -178,11 +177,11 @@ class ExceptionHandler(Object):
 
 		# Write the error page out to the response if available:
 		if self._res and (not self._res.isCommitted()
-			or self._res.header('Content-type', None)=='text/html'):
+			or self._res.header('Content-type', None) == 'text/html'):
 			if not self._res.isCommitted():
 				self._res.reset()
 				self._res.setStatus(500, "Servlet Error")
-			if self.setting('ShowDebugInfoOnErrors') ==1:
+			if self.setting('ShowDebugInfoOnErrors') == 1:
 				publicErrorPage = self.privateErrorPage()
 			else:
 				publicErrorPage = self.publicErrorPage()
@@ -204,7 +203,10 @@ class ExceptionHandler(Object):
 		if self.setting('EmailErrors'):
 			if privateErrorPage is None:
 				privateErrorPage = self.privateErrorPage()
-			self.emailException(privateErrorPage)
+			try:
+				self.emailException(privateErrorPage)
+			except Exception, e:
+				print "Could not send error email:", e
 
 	def logExceptionToConsole(self, stderr=None):
 		"""Log an exception.
@@ -217,7 +219,7 @@ class ExceptionHandler(Object):
 		if stderr is None:
 			stderr = sys.stderr
 		stderr.write('[%s] [error] WebKit: Error while executing script %s\n'
-			% (asctime(localtime(self._time)), self.servletPathname()))
+			% (asclocaltime(self._time), self.servletPathname()))
 		traceback.print_exc(file=stderr)
 
 	def publicErrorPage(self):
@@ -256,10 +258,10 @@ class ExceptionHandler(Object):
 		Calls `writeHTML`, which uses ``self.write(...)`` to add content.
 
 		"""
-		self.html = []
+		self._html = []
 		self.writeHTML()
-		html = ''.join(self.html)
-		self.html = None
+		html = ''.join(self._html)
+		self._html = None
 		return html
 
 	def writeHTML(self):
@@ -286,12 +288,12 @@ class ExceptionHandler(Object):
 
 	def write(self, s):
 		"""Output `s` to the body."""
-		self.html.append(str(s))
+		self._html.append(str(s))
 
 	def writeln(self, s):
 		"""Output `s` plus a newline."""
-		self.html.append(str(s))
-		self.html.append('\n')
+		self._html.append(str(s))
+		self._html.append('\n')
 
 	def writeTitle(self, s):
 		"""Output the sub-heading to define a section."""
@@ -351,11 +353,11 @@ class ExceptionHandler(Object):
 		"""
 		rows = []
 		for name in attrNames:
-			value = getattr(obj, '_'+name, singleton) # go for data attribute
+			value = getattr(obj, '_' + name, Singleton) # go for data attribute
 			try:
-				if value is singleton:
-					value = getattr(obj, name, singleton) # go for method
-					if value is singleton:
+				if value is Singleton:
+					value = getattr(obj, name, Singleton) # go for method
+					if value is Singleton:
 						value = '(could not find attribute or method)'
 					else:
 						try:
@@ -396,7 +398,7 @@ class ExceptionHandler(Object):
 		"""
 		self.writeTitle('MiscInfo')
 		info = {
-			'time':        asctime(localtime(self._time)),
+			'time':        asclocaltime(self._time),
 			'filename':    self.servletPathname(),
 			'os.getcwd()': os.getcwd(),
 			'sys.path':    sys.path,
@@ -446,12 +448,12 @@ class ExceptionHandler(Object):
 					import ExpansiveHTMLForException
 				self.write(ExpansiveHTMLForException(
 					context=self.setting('FancyTracebackContext')))
-			except:
+			except Exception:
 				self.write('<p>Unable to generate a fancy traceback!'
 					' (uncaught exception)</p>')
 				try:
 					self.write(HTMLForException(sys.exc_info()))
-				except:
+				except Exception:
 					self.write('<p>Unable to even generate a normal traceback'
 						' of the exception in fancy traceback!</p>')
 
@@ -477,7 +479,7 @@ class ExceptionHandler(Object):
 
 		"""
 		return 'Error-%s-%s-%d.html' % (self.basicServletName(),
-			'-'.join(map(lambda x: '%02d' % x, localtime(self._time)[:6])),
+			'-'.join(map(lambda x: '%02d' % x, time.localtime(self._time)[:6])),
 				random.randint(10000, 99999))
 			# @@ 2000-04-21 ce: Using the timestamp & a
 			# random number is a poor technique for
@@ -495,7 +497,7 @@ class ExceptionHandler(Object):
 		if not self.setting('LogErrors'):
 			return
 		logline = (
-			asctime(localtime(self._time)),
+			asclocaltime(self._time),
 			self.basicServletName(),
 			self.servletPathname(),
 			str(self._exc[0]),
@@ -525,18 +527,17 @@ class ExceptionHandler(Object):
 		or in the body of the mail.
 
 		"""
-		message = StringIO.StringIO()
+		message = StringIO()
 		writer = MimeWriter.MimeWriter(message)
 
 		# Construct the message headers
 		headers = self.setting('ErrorEmailHeaders').copy()
 		headers['Date'] = dateForEmail()
 		headers['Mime-Version'] = '1.0'
-		headers['Subject'] = headers.get('Subject','[WebKit Error]') + ' ' \
-					 + str(sys.exc_info()[0]) + ': ' \
-					 + str(sys.exc_info()[1])
-		for h,v in headers.items():
-			if isinstance(v, types.ListType):
+		headers['Subject'] = headers.get('Subject', '[WebKit Error]') \
+			+ ' %s: %s' % sys.exc_info()[:2]
+		for h, v in headers.items():
+			if isinstance(v, ListType):
 				v = ','.join(v)
 			writer.addheader(h, v)
 
@@ -546,16 +547,13 @@ class ExceptionHandler(Object):
 			# start off with a text/plain part
 			part = writer.nextpart()
 			body = part.startbody('text/plain')
-			body.write(
-				'WebKit caught an exception while processing ' +
-				'a request for "%s" ' % self.servletPathname() +
-				'at %s (timestamp: %s).  ' %
-				(asctime(localtime(self._time)), self._time) +
-				'The plain text traceback from Python is printed below and ' +
-				'the full HTML error report from WebKit is attached.\n\n'
-				)
+			body.write('WebKit caught an exception while processing'
+				' a request for "%s" at %s (timestamp: %s).'
+				' The plain text traceback from Python is printed below and'
+				' the full HTML error report from WebKit is attached.\n\n'
+					% (self.servletPathname(),
+					asclocaltime(self._time), self._time))
 			traceback.print_exc(file=body)
-
 			# now add htmlErrMsg
 			part = writer.nextpart()
 			part.addheader('Content-Transfer-Encoding', '7bit')
@@ -563,7 +561,6 @@ class ExceptionHandler(Object):
 				'HTML version of WebKit error message')
 			body = part.startbody('text/html; name=WebKitErrorMsg.html')
 			body.write(htmlErrMsg)
-
 			# finish off
 			writer.lastpart()
 		else:
@@ -572,17 +569,31 @@ class ExceptionHandler(Object):
 			body.write(htmlErrMsg)
 
 		# Send the message
-		server = self.setting('ErrorEmailServer') # can be: server, server:port, server:port:user:password
-		parts = server.split(':')
-		if len(parts)==1:
-			server = smtplib.SMTP(parts[0])
+		server = self.setting('ErrorEmailServer')
+		# this setting can be: server, server:port, server:port:user:password
+		parts = server.split(':', 3)
+		server = parts[0]
+		try:
+			port = int(parts[1])
+		except (IndexError, ValueError):
+			port = None
+		if port:
+			server = smtplib.SMTP(server, port)
 		else:
-			server = smtplib.SMTP(parts[0], int(parts[1]))  # server and port
+			server = smtplib.SMTP(server)
 		server.set_debuglevel(0)
-		if len(parts)==3:
-			server.login(parts[2], '')  # user and no password
-		elif len(parts)==4:
-			server.login(parts[2], parts[3])  # user and password
+		try:
+			user = parts[2]
+			try:
+				passwd = parts[3]
+			except IndexError:
+				passwd = ''
+			server.login(user, passwd)
+		except AttributeError: # Python < 2.2
+			raise smtplib.SMTPException, \
+				"Python version doesn't support SMTP authentication"
+		except IndexError:
+			pass
 		server.sendmail(headers['From'], headers['To'], message.getvalue())
 		server.quit()
 
@@ -606,7 +617,7 @@ class ExceptionHandler(Object):
 		Use the extra key, row and table args as necessary.
 
 		"""
-		if row.has_key('attr') and key!='attr':
+		if row.has_key('attr') and key != 'attr':
 			return self.filterValue(value, row['attr'])
 		else:
 			return self.filterValue(value, key)
@@ -615,17 +626,17 @@ class ExceptionHandler(Object):
 		"""Filter values.
 
 		This is the core filter method that is used in all filtering.
-		By default, it simply returns self.hiddenString if the key is
-		in self.hideValuesForField (case insensitive). Subclasses
+		By default, it simply returns self._hiddenString if the key is
+		in self._hideValuesForField (case insensitive). Subclasses
 		could override for more elaborate filtering techniques.
 
 		"""
 		try:
 			key = key.lower()
-		except:
+		except Exception:
 			pass
-		if key in self.hideValuesForFields:
-			return self.hiddenString
+		if key in self._hideValuesForFields:
+			return self._hiddenString
 		else:
 			return value
 
@@ -684,8 +695,8 @@ def osIdTable():
 
 	"""
 
-	funcs = ['getegid', 'geteuid', 'getgid', 'getpgrp', 'getpid',
-		 'getppid', 'getuid']
+	funcs = ['getegid', 'geteuid', 'getgid', 'getpgrp',
+		'getpid', 'getppid', 'getuid']
 	table = []
 	for funcName in funcs:
 		if hasattr(os, funcName):
