@@ -1,36 +1,25 @@
+import cgi, traceback
+from types import ListType
+
 from Common import *
-from WebUtils.Funcs import requestURI
 from WebUtils import FieldStorage
 from WebKit.Cookie import CookieEngine
 Cookie = CookieEngine.SimpleCookie
 from Request import Request
-from Session import Session, SessionError
 import HTTPResponse
 
-import os, cgi, sys, traceback
-from types import ListType
-
-
-debug=0
+debug = False
 
 
 class HTTPRequest(Request):
-	"""A type of Message for HTTP requests.
-
-	FUTURE
-		* How about some documentation?
-		* The "Information" section is a bit screwed up.
-		  Because the WebKit server adapter is a CGI script,
-		  these values are oriented towards that rather than the servlet.
-
-	"""
+	"""A type of Message for HTTP requests."""
 
 
 	## Initialization ##
 
-	def __init__(self, dict={}):
+	def __init__(self, dict=None):
 		Request.__init__(self)
-		self._parents = []
+		self._stack = []
 		if dict:
 			# Dictionaries come in from web server adapters like the CGIAdapter
 			assert dict['format'] == 'CGI'
@@ -38,52 +27,71 @@ class HTTPRequest(Request):
 			self._environ = dict['environ']
 			self._input = dict['input']
 			self._requestID = dict['requestID']
-			self._fields = FieldStorage.FieldStorage(self._input,
-				environ=self._environ, keep_blank_values=1, strict_parsing=0)
+			self._fields = FieldStorage.FieldStorage(
+				self._input, environ=self._environ,
+				keep_blank_values=True, strict_parsing=False)
 			self._fields.parse_qs()
 			self._cookies = Cookie()
 			if self._environ.has_key('HTTP_COOKIE'):
 				# Protect the loading of cookies with an exception handler,
-				# because some cookies from MSIE are known to break the cookie module.
+				# because MSIE cookies sometimes can break the cookie module.
 				try:
 					self._cookies.load(self._environ['HTTP_COOKIE'])
-				except:
+				except Exception:
 					traceback.print_exc(file=sys.stderr)
 		else:
-			# If there's no dictionary, we pretend we're a CGI script and see what happens...
-			import time
+			# If there's no dictionary, we pretend we're a CGI script
+			# and see what happens...
 			self._time = time.time()
 			self._environ = os.environ.copy()
 			self._input = None
-			self._fields = cgi.FieldStorage(keep_blank_values=1)
+			self._fields = cgi.FieldStorage(keep_blank_values=True)
 			self._cookies = Cookie()
 
+		env = self._environ
+
 		# Debugging
 		if debug:
 			f = open('env.text', 'a')
 			save = sys.stdout
 			sys.stdout = f
 			print '>> env for request:'
-			keys = self._environ.keys()
+			keys = env.keys()
 			keys.sort()
 			for key in keys:
-				print '%s: %s' % (repr(key), repr(self._environ[key]))
+				print '%s: %s' % (repr(key), repr(env[key]))
 			print
 			sys.stdout = save
 			f.close()
 
-		# Fix up environ if it doesn't look right.
-
-		# Fix #1: No PATH_INFO
-		# This can happen when there is no extra path info past the adapter.
-		# e.g., http://localhost/WebKit.cgi
-		if not self._environ.has_key('PATH_INFO'):
-			self._environ['PATH_INFO'] = ''
-		# Fix #2: No REQUEST_URI
-		# REQUEST_URI isn't actually part of the CGI standard and some
-		# web servers like IIS don't set it (as of 8/22/2000).
-		if not self._environ.has_key('REQUEST_URI'):
-			self._environ['REQUEST_URI'] = requestURI(self._environ)
+		# Get adapter, servlet path and query string
+		self._servletPath = env.get('SCRIPT_NAME', '')
+		self._pathInfo = env.get('PATH_INFO', '')
+		self._queryString = env.get('QUERY_STRING', '')
+		if env.has_key('REQUEST_URI'):
+			self._uri = env['REQUEST_URI']
+			# correct servletPath if there was a redirection
+			if not (self._uri + '/').startswith(self._servletPath + '/'):
+				i = self._uri.find(self._pathInfo)
+				if i >= 0:
+					self._servletPath = self._uri[:i]
+		else:
+			# REQUEST_URI isn't actually part of the CGI standard and some
+			# web servers like IIS don't set it (as of 8/22/2000).
+			if env.has_key('SCRIPT_URL'):
+				self._uri = self._environ['SCRIPT_URL']
+				# correct servletPath if there was a redirection
+				if not (self._uri + '/').startswith(self._servletPath + '/'):
+					i = self._uri.find(self._pathInfo)
+					if i >= 0:
+						self._servletPath = self._uri[:i]
+			else:
+				self._uri = self._servletPath + self._pathInfo
+			if self._queryString:
+				self._uri += '?' + self._queryString
+		self._absolutepath = env.has_key('WK_ABSOLUTE') # set by adapter
+		if self._absolutepath:
+			self._fsPath = self.fsPath()
 
 		# We use the cgi module to get the fields,
 		# but then change them into an ordinary dictionary of values:
@@ -119,17 +127,12 @@ class HTTPRequest(Request):
 			dict[key] = self._cookies[key].value
 		self._cookies = dict
 
-		self._transaction = None
+		self._contextName = None
+		self._serverSidePath = self._serverSideContextPath = None
 		self._serverRootPath = self._extraURLPath = ''
-		self._cookieSession = self._pathSession = None
-		self._sessionExpired = 0
+		self._sessionExpired = False
 
-		self._absolutepath = self._environ.has_key('WK_ABSOLUTE') # set by the adapter
-		if self._absolutepath:
-			self._fsPath = self.fsPath()
-		self._adapterName = self.servletPath()
 		self._pathInfo = self.pathInfo()
-		self._originalURLPath = self.urlPath()
 
 		if debug:
 			print "Done setting up request, found keys %r" % self._fields.keys()
@@ -140,36 +143,6 @@ class HTTPRequest(Request):
 	def responseClass(self):
 		return HTTPResponse.HTTPResponse
 
-	def transaction(self):
-		return self._transaction
-
-	def setTransaction(self, trans):
-		"""Should be invoked after the transaction is created for this request."""
-		self._transaction = trans
-		self._sidname = trans.application()._session_name
-		# Get session ID from cookie if available
-		self._cookieSession = self._cookies.get(self._sidname, None)
-		# try to get automatic path session
-		# if UseAutomaticPathSessions is enabled in Application.config
-		# Application.py redirects the browser to a url with SID in path
-		# http://gandalf/a/_SID_=2001080221301877755/Examples/
-		# _SID_ is extracted and removed from path
-		p = self._pathInfo
-		if p:
-			p = p.split('/', 2)
-			if len(p) > 1 and not p[0] and p[1].startswith(self._sidname + '='):
-				self._pathSession = p[1].split('=', 1)[1]
-				self._cookies[self._sidname] = self._pathSession
-				sidpart = p[1] + '/'
-				del p[1]
-				self._environ['PATH_INFO'] = '/'.join(p)
-				if self._environ.has_key('REQUEST_URI'):
-					self._environ['REQUEST_URI'] = self._environ[
-						'REQUEST_URI'].replace(sidpart, '')
-				if self._environ.has_key('PATH_TRANSLATED'):
-					self._environ['PATH_TRANSLATED'] = self._environ[
-						'PATH_TRANSLATED'].replace(sidpart, '')
-
 
 	## Values ##
 
@@ -226,10 +199,11 @@ class HTTPRequest(Request):
 			return self._cookies.get(name, default)
 
 	def hasCookie(self, name):
+		"""Return whether a cookie with the given name exists."""
 		return self._cookies.has_key(name)
 
 	def cookies(self):
-		"""Return a dictionary-style object of all Cookie objects the client sent with this request."""
+		"""Return a dict of all cookies the client sent with this request."""
 		return self._cookies
 
 
@@ -256,11 +230,11 @@ class HTTPRequest(Request):
 		return self._transaction.session()
 
 	def isSessionExpired(self):
-		"""Return whether or not this request originally contained an expired session ID.
+		"""Return whether the request originally had an expired session ID.
 
-		Only works if the Application.config setting "IgnoreInvalidSession" is set to 1;
-		otherwise you get a canned error page on an invalid session,
-		so your servlet never gets processed.
+		Only works if the Application.config setting "IgnoreInvalidSession"
+		is set to true; otherwise you get a canned error page on an invalid
+		session, so your servlet never gets processed.
 
 		"""
 		return self._sessionExpired
@@ -280,7 +254,7 @@ class HTTPRequest(Request):
 		return self._environ['REMOTE_USER']
 
 	def remoteAddress(self):
-		"""Return a string containing the IP address of the client that sent the request."""
+		"""Return a string containing the IP address of the client."""
 		return self._environ['REMOTE_ADDR']
 
 	def remoteName(self):
@@ -293,7 +267,7 @@ class HTTPRequest(Request):
 		return env.get('REMOTE_NAME', env['REMOTE_ADDR'])
 
 	def accept(self, which=None):
-		"""Returns preferences as requested by the user agent.
+		"""Return preferences as requested by the user agent.
 
 		The accepted preferences are returned as a list of codes
 		in the same order as they appeared in the header.
@@ -317,9 +291,10 @@ class HTTPRequest(Request):
 	## Path ##
 
 	def urlPath(self):
-		"""Return the URL path of the servlet sans host, adapter and query string.
+		"""Return URL path without host, adapter and query string.
 
-		For example, http://host/WebKit.cgi/Context/Servlet?x=1 yields '/Context/Servlet'.
+		For example, http://host/WebKit.cgi/Context/Servlet?x=1
+		yields '/Context/Servlet'.
 
 		If self._absolutepath is set, this refers to the filesystem path.
 
@@ -329,10 +304,6 @@ class HTTPRequest(Request):
 		else:
 			return self._pathInfo
 
-	def originalURLPath(self):
-		"""Return the URL path of the _original_ servlet before any forwarding."""
-		return self._originalURLPath
-
 	def urlPathDir(self):
 		"""Same as urlPath, but only gives the directory."""
 		return os.path.dirname(self.urlPath())
@@ -348,8 +319,10 @@ class HTTPRequest(Request):
 		if self._absolutepath:
 			self._fsPath = path
 		else:
-			self._pathInfo = self._environ['PATH_INFO'] = path
-			self._environ['REQUEST_URI'] = self._adapterName + path
+			self._pathInfo = path
+			self._uri = self._servletPath + path
+			if self._queryString:
+				self._uri += '?' + self._queryString
 
 	def serverSidePath(self, path=None):
 		"""Return the absolute server-side path of the request.
@@ -369,8 +342,8 @@ class HTTPRequest(Request):
 	def serverSideContextPath(self, path=None):
 		"""Return the absolute server-side path of the context of this request.
 
-		If the optional path is passed in, then it is joined with the server side
-		context directory to form a path relative to the object.
+		If the optional path is passed in, then it is joined with the server
+		side context directory to form a path relative to the object.
 
 		This directory could be different from the result of serverSidePath()
 		if the request is in a subdirectory of the main context directory.
@@ -387,13 +360,14 @@ class HTTPRequest(Request):
 	def contextName(self):
 		"""Return the name of the context of this request.
 
-		This isn't necessarily the same as the name of the directory containing the context.
+		This isn't necessarily the same as the name of the directory
+		containing the context.
 
 		"""
 		return self._contextName
 
 	def servletURI(self):
-		"""Return the URI of the servlet, without any query strings or extra path info."""
+		"""Return servlet URI without any query strings or extra path info."""
 		p = self._pathInfo
 		if not self._extraURLPath:
 			if p.endswith('/'):
@@ -407,6 +381,7 @@ class HTTPRequest(Request):
 		return p
 
 	def uriWebKitRoot(self):
+		"""Return relative URL path of the WebKit root location."""
 		if not self._serverRootPath:
 			self._serverRootPath = ''
 			loc = self.urlPath()
@@ -420,7 +395,7 @@ class HTTPRequest(Request):
 
 	def fsPath(self):
 		"""The filesystem path of the request according to the webserver."""
-		fspath = self.servletFilePath()
+		fspath = self.adapterFileName()
 		if not fspath:
 			fspath = self.servletPath()
 			docroot = self._environ['DOCUMENT_ROOT']
@@ -430,21 +405,21 @@ class HTTPRequest(Request):
 	def serverURL(self):
 		"""Return the full internet path to this request.
 
-		This is the URL that was actually received by the
-		webserver before any rewriting took place.
+		This is the URL that was actually received by the webserver
+		before any rewriting took place.
 
 		The path is returned without any extra path info or query strings,
 		i.e. http://www.my.own.host.com:8080/WebKit/TestPage.py
 
 		"""
-		url = self._environ.get('SCRIPT_URI', None)
-		if not url:
+		if self._environ.has_key('SCRIPT_URI'):
+			return self._environ['SCRIPT_URI']
+		else:
 			scheme = 'http'
 			if self._environ.get('HTTPS', '').lower() == 'on':
 				scheme += 's'
 			host = self._environ['HTTP_HOST'] # includes port
-			url = scheme + '://' + host + self.serverPath()
-		return url
+			return scheme + '://' + host + self.serverPath()
 
 	def serverURLDir(self):
 		"""Return the directory of the URL in full internet form.
@@ -460,16 +435,16 @@ class HTTPRequest(Request):
 	def serverPath(self):
 		"""Return the webserver URL path of this request.
 
-		This is the URL that was actually received by the
-		webserver before any rewriting took place.
+		This is the URL that was actually received by the webserver
+		before any rewriting took place.
 
 		Same as serverURL, but without scheme and host.
 
 		"""
-		url = self._environ.get('SCRIPT_URL', None)
-		if not url:
-			url = self._adapterName + self._pathInfo
-		return url
+		if self._environ.has_key('SCRIPT_URL'):
+			return self._environ['SCRIPT_URL']
+		else:
+			return self._servletPath + self._pathInfo
 
 	def serverPathDir(self):
 		"""Return the directory of the webserver URL path.
@@ -483,7 +458,10 @@ class HTTPRequest(Request):
 		return fullurl
 
 	def siteRoot(self):
-		"""Return the URL path components necessary to get back home from the current location.
+		"""Return the relative URL path of the home location.
+
+		This includes all URL path components necessary to get back home
+		from the current location.
 
 		Examples:
 			''
@@ -492,12 +470,12 @@ class HTTPRequest(Request):
 
 		You can use this as a prefix to a URL that you know is based off
 		the home location. Any time you are in a servlet that may have been
-		forwarded to from another servlet at a different level,
-		you should prefix your URL's with this. That is, if servlet "Foo/Bar"
-		forwards to "Qux", then the qux servlet should use siteRoot() to construct all
+		forwarded to from another servlet at a different level, you should
+		prefix your URL's with this. That is, if servlet "Foo/Bar" forwards
+		to "Qux", then the qux servlet should use siteRoot() to construct all
 		links to avoid broken links. This works properly because this method
-		computes the path based on the _original_ servlet, not the location of the
-		servlet that you have forwarded to.
+		computes the path based on the _original_ servlet, not the location
+		of the servlet that you have forwarded to.
 
 		"""
 		url = self.originalURLPath()
@@ -510,7 +488,10 @@ class HTTPRequest(Request):
 		return '../' * numStepsBack
 
 	def siteRootFromCurrentServlet(self):
-		"""Return the URL path componentes necessary to get back home from the current servlet.
+		"""Return relative URL path to home seen from the current servlet.
+
+		This includes all URL path components necessary to get back home
+		from the current servlet (not from the original request).
 
 		Similar to siteRoot() but instead, it returns the site root
 		relative to the _current_ servlet, not the _original_ servlet.
@@ -529,9 +510,9 @@ class HTTPRequest(Request):
 		"""Return the "servlet path" of this servlet relative to the siteRoot.
 
 		In other words, everything after the name of the context (if present).
-		If you append this to the result of self.siteRoot() you get back to the
-		current servlet. This is useful for saving the path to the current servlet
-		in a database, for example.
+		If you append this to the result of self.siteRoot() you get back to
+		the current servlet. This is useful for saving the path to the current
+		servlet in a database, for example.
 
 		"""
 		urlPath = self.urlPath()
@@ -554,55 +535,117 @@ class HTTPRequest(Request):
 	def adapterName(self):
 		"""Return the name of the adapter as it appears in the URL.
 
-		Example: '/WebKit.cgi'
-		This is useful in special cases when you are constructing URLs.
-		See Testing/Main.py for an example use.
+		Example: '/WK' or '/WebKit.cgi'
+		Does not reflect redirection by the webserver.
+		Equivalent to the CGI variable SCRIPT_NAME.
 
 		"""
-		return self._adapterName
+		return self._environ.get('SCRIPT_NAME', '')
 
-	def rawRequest(self):
-		"""Return the raw request that was used to initialize this request object."""
-		return self._rawRequest
+	def adapterFileName(self):
+		"""Return the filesystem path of the adapter.
+
+		Equivalent to the CGI variable SCRIPT_FILENAME.
+
+		"""
+		return self._environ.get('SCRIPT_FILENAME', '')
 
 	def environ(self):
+		"""Get the environment for the request."""
 		return self._environ
 
-	def addParent(self, servlet):
-		self._parents.append(servlet)
+	def push(self, servlet, url=None):
+		"""Push servlet and URL path on a stack, setting a new URL."""
+		self._stack.append((servlet, self.urlPath(), self._contextName,
+			self._serverSidePath, self._serverSideContextPath,
+			self._serverRootPath, self._extraURLPath))
+		if url is not None:
+			self.setURLPath(url)
 
-	def popParent(self):
-		if self._parents:
-			self._parents.pop()
+	def pop(self):
+		"""Pop URL path and servlet from the stack, returning the servlet."""
+		if self._stack:
+			(servlet, url, self._contextName,
+				self._serverSidePath, self._serverSideContextPath,
+				self._serverRootPath, self._extraURLPath) = self._stack.pop()
+			if url is not None:
+				self.setURLPath(url)
+			return servlet
 
-	def parent(self):
+	def servlet(self):
+		"""Get current servlet for this request."""
+		return self._transaction.servlet()
+
+	def originalServlet(self):
+		"""Get original servlet before any forwarding."""
+		if self._stack:
+			return self._stack[0][0]
+		else:
+			self.servlet()
+
+	def previousServlet(self):
 		"""Get the servlet that passed this request to us, if any."""
-		if self._parents:
-			return self._parents[len(self._parents)-1]
+		if self._stack:
+			return self._stack[-1][0]
 
-	def parents(self):
-		"""Return the parents list."""
-		return self._parents
+	parent = previousServlet # kept old name as synonym
 
-	def rawInput(self, rewind=0):
+	def previousServlets(self):
+		"""Get the list of all previous servlets."""
+		return [s[0] for s in self._stack]
+
+	parents = previousServlets # kept old name as synonym
+
+	def originalURLPath(self):
+		"""Get URL path of the original servlet before any forwarding."""
+		if self._stack:
+			return self._stack[0][1]
+		else:
+			return self.urlPath()
+
+	def previousURLPath(self):
+		"""Get the previous URL path, if any."""
+		if self._stack:
+			return self._stack[-1][1]
+
+	def previousURLPaths(self):
+		"""Get the list of all previous URL paths."""
+		return [s[1] for s in self._stack]
+
+	def originalURI(self):
+		"""Get URI of the original servlet before any forwarding."""
+		if self._stack:
+			return self._stack[0][2]
+		else:
+			return self.uri()
+
+	def previousURI(self):
+		"""Get the previous URI, if any."""
+		if self._stack:
+			return self._stack[-1][2]
+
+	def previousURIs(self):
+		"""Get the list of all previous URIs."""
+		return [s[2] for s in self._stack]
+
+	def rawInput(self, rewind=False):
 		"""Get the raw input from the request.
 
-		This gives you a file-like object for the data that was
-		sent with the request (e.g., the body of a POST request,
-		or the documented uploaded in a PUT request).
+		This gives you a file-like object for the data that was sent with
+		the request (e.g., the body of a POST request, or the document
+		uploaded in a PUT request).
 
-		The file might not be rewound to the beginning if there
-		was valid, form-encoded POST data. Pass rewind=1 if
-		you want to be sure you get the entire body of the request.
+		The file might not be rewound to the beginning if there was valid,
+		form-encoded POST data. Pass rewind=True if you want to be sure
+		you get the entire body of the request.
 
 		"""
 		fs = self.fieldStorage()
-		if fs and fs.file:
-			if rewind:
-				fs.file.seek(0)
-			return fs.file
-		else:
+		if fs is None:
 			return None
+		if rewind and fs.file:
+			fs.file.seek(0)
+		return fs.file
 
 	def time(self):
 		"""Return the time that the request was received."""
@@ -620,33 +663,36 @@ class HTTPRequest(Request):
 
 	## Information ##
 
-	# @@ 2000-05-10: See FUTURE section of class doc string
-
 	def servletPath(self):
-		"""Return the URI of the script, sans host."""
-		return self._environ.get('SCRIPT_NAME', '')
+		"""Return the base URL for the servlets, sans host.
 
-	def servletFilePath(self):
-		"""Return the filesystem path of the script."""
-		return self._environ.get('SCRIPT_FILENAME', '')
+		This is useful in cases when you are constructing URLs.
+		See Testing/Main.py for an example use.
+
+		Roughly equivalent to the CGI variable SCRIPT_NAME,
+		but reflects redirection by the webserver.
+
+		"""
+		return self._servletPath
 
 	def contextPath(self):
-		"""Return the portion of the request URI that is the context of the request."""
+		"""Return the portion of the URI that is the context of the request."""
 		return self._serverSideContextPath
 
 	def pathInfo(self):
-		"""Return any extra path information associated with the URL the client sent with this request.
+		"""Return any extra path information as sent by the client.
 
-		Equivalent to CGI variable PATH_INFO.
+		This is anything after the servlet name but before the query string.
+		Equivalent to the CGI variable PATH_INFO.
 
 		"""
-		return self._environ.get('PATH_INFO', '')
+		return self._pathInfo
 
 	def pathTranslated(self):
-		"""Return any extra path information after the servlet name but before the query string.
+		"""Return extra path information translated as file system path.
 
-		The path is translated to a file system path.
-		Equivalent to CGI variable PATH_TRANSLATED.
+		This is the same as pathInfo() but translated to the file system.
+		Equivalent to the CGI variable PATH_TRANSLATED.
 
 		"""
 		return self._environ.get('PATH_TRANSLATED', '')
@@ -654,19 +700,20 @@ class HTTPRequest(Request):
 	def queryString(self):
 		"""Return the query string portion of the URL for this request.
 
-		Taken from the CGI variable QUERY_STRING.
+		Equivalent to the CGI variable QUERY_STRING.
 
 		"""
-		return self._environ.get('QUERY_STRING', '')
+		return self._queryString
 
 	def uri(self):
 		"""Return the URI for this request (everything after the host name).
 
-		Taken from the CGI variable REQUEST_URI. If not available,
-		constructed from SCRIPT_NAME, PATH_INFO and QUERY_STRING.
+		This is the URL that was actually received by the webserver
+		before any rewriting took place, including the query string.
+		Equivalent to the CGI variable REQUEST_URI.
 
 		"""
-		return requestURI(self._environ)
+		return self._uri
 
 	def method(self):
 		"""Return the HTTP request method (in all uppercase).
@@ -682,8 +729,10 @@ class HTTPRequest(Request):
 		Returns None if there is no session ID.
 
 		"""
-		sid = self.value(self._sidname, None)
-		if self._transaction.application().setting('Debug')['Sessions']:
+		trans = self._transaction
+		app = trans.application()
+		sid = self.value(app.sessionName(trans), None)
+		if app.setting('Debug')['Sessions']:
 			print '>> sessionId: returning sid =', sid
 		return sid
 
@@ -707,18 +756,13 @@ class HTTPRequest(Request):
 		"""
 		# Modify the request so that it looks like a hashed version of the
 		# given session ID was passed in
-		self.setField(self._sidname, sessionID)
-
+		trans = self._transaction
+		app = trans.application()
+		self.setField(app.sessionName(trans), sessionID)
 		if force:
 			# Force this session ID to exist, so that a random session ID
 			# won't be created in case it's a new session.
-			self._transaction.application().createSessionWithID(self._transaction, sessionID)
-
-	def hasPathSession(self):
-		return self._pathSession is not None
-
-	def hasCookieSession(self):
-		return self._cookieSession is not None
+			app.createSessionWithID(trans, sessionID)
 
 
 	## Inspection ##
@@ -729,8 +773,8 @@ class HTTPRequest(Request):
 		Return a list of tuples where each tuple has a key/label (a string)
 		and a value (any kind of object).
 
-		Values are typically atomic values such as numbers and strings
-		or another list of tuples in the same fashion. This is for debugging only.
+		Values are typically atomic values such as numbers and strings or
+		another list of tuples in the same fashion. This is for debugging only.
 
 		"""
 		# @@ 2000-04-10 ce: implement and invoke super if appropriate
@@ -747,8 +791,8 @@ class HTTPRequest(Request):
 		# Information methods
 		for method in _infoMethods:
 			try:
-				info.append((method.__name__, apply(method, (self,))))
-			except:
+				info.append((method.__name__, method(self)))
+			except Exception:
 				info.append((method.__name__, None))
 
 		return info
@@ -760,21 +804,11 @@ class HTTPRequest(Request):
 
 		"""
 		return htmlInfo(self.info())
-		info = self.info()
-		res = ['<table border="1">\n']
-		for pair in info:
-			value = pair[1]
-			if hasattr(value, 'items') and (type(value) is type({})
-					or hasattr(value, '__getitem__')):
-				value = _infoForDict(value)
-			res.append('<tr valign="top"><td>%s</td><td>%s&nbsp;</td></tr>\n'
-				% (pair[0], value))
-		res.append('</table>\n')
-		return ''.join(res)
 
-	exceptionReportAttrNames = Request.exceptionReportAttrNames + (
-		'uri servletPath serverSidePath pathInfo pathTranslated'
-		' queryString method sessionId parents fields cookies environ'.split())
+	_exceptionReportAttrNames = Request._exceptionReportAttrNames + (
+		'uri adapterName servletPath serverSidePath'
+		' pathInfo pathTranslated queryString method'
+		' sessionId previousURLPaths fields cookies environ'.split())
 
 
 	## Deprecated ##
@@ -784,7 +818,7 @@ class HTTPRequest(Request):
 
 		Use serverSidePath() instead.@
 
-		Return the directory of the Servlet (as given through __init__()'s path).
+		Return the servlet directory (as given through __init__()'s path).
 
 		"""
 		self.deprecated(self.serverSideDir)
@@ -797,17 +831,29 @@ class HTTPRequest(Request):
 
 		Use serverSidePath() instead.@
 
-		Return a new path which includes the servlet's path appended by 'joinPath'.
-		Note that if 'joinPath' is an absolute path, then only 'joinPath' is returned.
+		Return a new path with the servlet's path appended by 'joinPath'.
+		If 'joinPath' is an absolute path, then only 'joinPath' is returned.
 
 		"""
 		self.deprecated(self.relativePath)
 		return os.path.join(self.serverSideDir(), joinPath)
 
+	def servletFilePath(self):
+		"""deprecated: HTTPRequest.servletFilePath() on 04/12/07 in 0.9.3.
+
+		Use adapterFileName() instead.@
+
+		Equivalent to the CGI variable SCRIPT_FILENAME.
+
+		"""
+		self.deprecated(self.servletFilePath)
+		return self.adapterFileName(file)
+
 
 ## Info Structure ##
 
 _infoMethods = (
+	HTTPRequest.adapterName,
 	HTTPRequest.servletPath,
 	HTTPRequest.contextPath,
 	HTTPRequest.pathInfo,
@@ -818,7 +864,6 @@ _infoMethods = (
 	HTTPRequest.sessionId
 )
 
-
 def htmlInfo(info):
 	"""Return a single HTML string that represents the info structure.
 
