@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+
 """Threaded Application Server
 
 The AppServer is the main process of WebKit. It handles requests for
@@ -17,38 +18,39 @@ stop: stop the currently running Apperver
 daemon: run as a daemon
 ClassName.SettingName=value: change configuration settings
 
-When started, AppServer records its pid in appserverpid.txt.
+When started, the app server records its pid in appserver.pid.
 
 """
 
+from marshal import dumps, loads
+import threading, Queue, select, socket, errno, traceback
+
 from Common import *
 import AppServer as AppServerModule
+from PidFile import ProcessRunning
 from AutoReloadingAppServer import AutoReloadingAppServer as AppServer
+from ASStreamOut import ASStreamOut, ConnectionAbortedError
 from MiscUtils.Funcs import timestamp
-from WebUtils import Funcs
+from WebUtils.Funcs import requestURI
 
-from marshal import dumps, loads
-import os, sys, threading, Queue, select, socket, time, errno, traceback
+debug = False
 
-
-debug = 0
-
-DefaultConfig = {
-	'Host':                 '127.0.0.1',
-	'EnableAdapter':        True,
-	'AdapterPort':          8086,
-	'EnableMonitor':        False,
-	'MonitorPort':          8085,
-	'EnableHTTP':           False,
-	'HTTPPort':             8080,
-	'MaxServerThreads':     20,
-	'MinServerThreads':     5,
-	'StartServerThreads':   10,
-
-	# @@ 2000-04-27 ce: None of the following settings are implemented
-	# 'RequestQueueSize': 16,
-	# 'RequestBufferSize': 64*1024,
-	# 'SocketType': 'inet', # inet, unix
+defaultConfig = {
+	'Host': 'localhost', # same as '127.0.0.1'
+	'EnableAdapter': True, # enable WebKit adapter
+	'AdapterPort': 8086,
+	'EnableMonitor': False, # disable status monitoring
+	'MonitorPort': 8085,
+	'EnableHTTP': True, # enable built-in HTTP server
+	'HTTPPort': 8080,
+	'StartServerThreads': 10, # initial number of server threads
+	'MinServerThreads': 5, # minimum number
+	'MaxServerThreads': 20, # maxium number
+	'RequestQueueSize': 0, # means twice the maximum number of threads
+	'RequestBufferSize': 8*1024, # 8 kBytes
+	'ResponseBufferSize': 8*1024, # 8 kBytes
+	# @@ the following setting is not yet implemented
+	# 'SocketType': 'inet', # inet, inet6, unix
 }
 
 # Need to know this value for communications
@@ -56,7 +58,9 @@ DefaultConfig = {
 # from the AppServer to 2,147,483,647 bytes):
 intLength = len(dumps(int(1)))
 
+# Initialize global variables
 server = None
+exitStatus = 0
 
 
 class NotEnoughDataError(Exception):
@@ -85,55 +89,57 @@ class ThreadedAppServer(AppServer):
 	## Init ##
 
 	def __init__(self, path=None):
-		"""
-		Setup the AppServer, create an initial thread pool
-		(threads created with `spawnThread`), record the PID
-		in a file, and add any enabled handlers (Adapter, HTTP, Monitor).
-		"""
+		"""Setup the AppServer.
 
+		Create an initial thread pool (threads created with `spawnThread`),
+		and the request queue, record the PID in a file, and add any enabled
+		handlers (Adapter, HTTP, Monitor).
+
+		"""
 		self._defaultConfig = None
-		try:
-			AppServer.__init__(self, path)
-			threadCount = self.setting('StartServerThreads')
-			self._maxServerThreads = self.setting('MaxServerThreads')
-			self._minServerThreads = self.setting('MinServerThreads')
-			self._threadPool = []
-			self._threadCount = 0
-			self._threadUseCounter = []
-			# twice the number of threads we have:
-			self._requestQueue = Queue.Queue(self._maxServerThreads * 2)
-			self._addr = {}
-			self._requestID = 0
+		AppServer.__init__(self, path)
 
-			out = sys.stdout
+		threadCount = self.setting('StartServerThreads')
+		self._maxServerThreads = self.setting('MaxServerThreads')
+		self._minServerThreads = self.setting('MinServerThreads')
+		self._requestQueueSize = self.setting('RequestQueueSize')
+		if not self._requestQueueSize:
+			# if not set, make queue size twice the max number of threads
+			self._requestQueueSize = 2 * self._maxServerThreads
+		elif self._requestQueueSize < self._maxServerThreads:
+			# otherwise do not make it smaller than the max number of threads
+			self._requestQueueSize = self._maxServerThreads
+		self._requestBufferSize = self.setting('RequestBufferSize')
+		self._responseBufferSize = self.setting('ResponseBufferSize')
 
-			out.write('Creating %d threads' % threadCount)
-			for i in range(threadCount):
-				self.spawnThread()
-				out.write(".")
-				out.flush()
-			out.write("\n")
+		self._threadPool = []
+		self._threadCount = 0
+		self._threadUseCounter = []
+		self._requestQueue = Queue.Queue(self._requestQueueSize)
+		self._addr = {}
+		self._requestID = 0
 
-			self._socketHandlers = {}
-			self._handlerCache = {}
-			self._sockets = {}
+		out = sys.stdout
+		out.write('Creating %d threads' % threadCount)
+		for i in range(threadCount):
+			self.spawnThread()
+			out.write(".")
+			out.flush()
+		out.write("\n")
 
-			if self.setting('EnableAdapter'):
-				self.addSocketHandler(AdapterHandler)
+		self._socketHandlers = {}
+		self._handlerCache = {}
+		self._sockets = {}
 
-			if self.setting('EnableMonitor'):
-				self.addSocketHandler(MonitorHandler)
+		if self.setting('EnableAdapter'):
+			self.addSocketHandler(AdapterHandler)
+		if self.setting('EnableMonitor'):
+			self.addSocketHandler(MonitorHandler)
+		if self.setting('EnableHTTP'):
+			from HTTPServer import HTTPAppServerHandler
+			self.addSocketHandler(HTTPAppServerHandler)
 
-			if self.setting('EnableHTTP'):
-				from WebKit.HTTPServer import HTTPAppServerHandler
-				self.addSocketHandler(HTTPAppServerHandler)
-
-			self.readyForRequests()
-		except:
-			if self.running:
-				self.initiateShutdown()
-				self._closeThread.join()
-			raise
+		self.readyForRequests()
 
 	def addSocketHandler(self, handlerClass, serverAddress=None):
 		"""Add socket handler.
@@ -166,27 +172,27 @@ class ThreadedAppServer(AppServer):
 		self._socketHandlers[serverAddress] = handlerClass
 		self._handlerCache[serverAddress] = []
 		self._sockets[serverAddress] = sock
-		adr_str = ':'.join(map(str, serverAddress))
-		print "Listening for %s on %s" % (handlerClass.settingPrefix, adr_str)
+		adrStr = ':'.join(map(str, serverAddress))
+		print "Listening for %s on %s" % (handlerClass.settingPrefix, adrStr)
 		# write text file with server address
-		adr_file = self.addressFileName(handlerClass)
-		if os.path.exists(adr_file):
-			print "Warning: %s already exists" % adr_file
+		adrFile = self.addressFileName(handlerClass)
+		if os.path.exists(adrFile):
+			print "Warning: %s already exists" % adrFile
 			try:
-				os.unlink(adr_file)
+				os.unlink(adrFile)
 			except OSError: # we cannot remove the file
-				if open(adr_file).read() == adr_str:
+				if open(adrFile).read() == adrStr:
 					return # same content, so never mind
 				else:
-					print "Error: Could not remove", adr_file
+					print "Error: Could not remove", adrFile
 					sys.stdout.flush()
 					raise
 		try:
-			f = open(adr_file, 'w')
-			f.write(adr_str)
+			f = open(adrFile, 'w')
+			f.write(adrStr)
 			f.close()
 		except IOError:
-			print "Error: Could not write", adr_file
+			print "Error: Could not write", adrFile
 			sys.stdout.flush()
 			raise
 
@@ -194,9 +200,12 @@ class ThreadedAppServer(AppServer):
 		return True
 
 	def defaultConfig(self):
+		"""The default AppServer.config."""
 		if self._defaultConfig is None:
 			self._defaultConfig = AppServer.defaultConfig(self).copy()
-			self._defaultConfig.update(DefaultConfig)
+			# Update with ThreadedAppServer specific settings
+			# as defined in defaultConfig on the module level:
+			self._defaultConfig.update(defaultConfig)
 		return self._defaultConfig
 
 	def mainloop(self, timeout=1):
@@ -205,10 +214,10 @@ class ThreadedAppServer(AppServer):
 		This is the main thread loop that accepts and dispatches
 		socket requests.
 
-		It goes through a loop as long as ``self.running > 2``.
-		Setting ``self.running = 2`` asks the the main loop to end.
-		When the main loop is finished, it sets ``self.running = 1``.
-		When the AppServer is completely down, it sets ``self.running = 0``.
+		It goes through a loop as long as ``self._running > 2``.
+		Setting ``self._running = 2`` asks the the main loop to end.
+		When the main loop is finished, it sets ``self._running = 1``.
+		When the AppServer is completely down, it sets ``self._running = 0``.
 
 		The loop waits for connections, then based on the connecting
 		port it initiates the proper Handler (e.g.,
@@ -225,14 +234,14 @@ class ThreadedAppServer(AppServer):
 
 		"""
 
-		threadCheckInterval = self._maxServerThreads*2
+		threadCheckInterval = self._maxServerThreads * 2
 		threadUpdateDivisor = 5 # grab stat interval
 		threadCheck = 0
 
-		self.running = 3 # server is in the main loop now
+		self._running = 3 # server is in the main loop now
 
 		try:
-			while self.running > 2:
+			while self._running > 2:
 
 				# block for timeout seconds waiting for connections
 				try:
@@ -268,7 +277,7 @@ class ThreadedAppServer(AppServer):
 				self.restartIfNecessary()
 
 		finally:
-			self.running = 1
+			self._running = 1
 
 
 	## Thread Management ##
@@ -310,9 +319,9 @@ class ThreadedAppServer(AppServer):
 
 		"""
 
-		# @@: This algorithm needs work. The edges (i.e. at
-		# the minserverthreads) are tricky. When working
-		# with this, remember thread creation is *cheap*.
+		# @@: This algorithm needs work. The edges (i.e. at the
+		# minserverthreads) are tricky. When working with this,
+		# remember thread creation is *cheap*.
 
 		average = max = 0
 
@@ -322,11 +331,11 @@ class ThreadedAppServer(AppServer):
 			average += i
 			if i > max:
 				max = i
-		average = average / len(self._threadUseCounter)
+		average /= len(self._threadUseCounter)
 		if debug:
-			print "Average Thread Use: ", avg
+			print "Average Thread Use: ", average
 			print "Max Thread Use: ", max
-			print "ThreadCount: ", self.threadCount
+			print "ThreadCount: ", self._threadCount
 
 		if len(self._threadUseCounter) < self._maxServerThreads:
 			return # not enough samples
@@ -395,7 +404,7 @@ class ThreadedAppServer(AppServer):
 				rv = i.join() # Don't need a timeout, it isn't alive
 				self._threadPool.remove(i)
 				if debug:
-					print "Thread absorbed, real threadCount =", len(self.threadPool)
+					print "Thread absorbed, real threadCount =", len(self._threadPool)
 
 
 	## Worker Threads ##
@@ -432,7 +441,7 @@ class ThreadedAppServer(AppServer):
 					t._processing = True
 					try:
 						handler.handleRequest()
-					except:
+					except Exception:
 						traceback.print_exc(file=sys.stderr)
 					t._processing = False
 					handler.close()
@@ -441,7 +450,7 @@ class ThreadedAppServer(AppServer):
 		finally:
 			self.delThread()
 		if debug:
-				print threading.currentThread(), "Quitting."
+			print threading.currentThread(), "Quitting."
 
 	def initThread(self):
 		"""Initialize thread.
@@ -471,13 +480,13 @@ class ThreadedAppServer(AppServer):
 		and tells all the threads to die.
 
 		"""
-		if self.running > 2:
-			self.running = 2 # ask main loop to finish
+		if self._running > 2:
+			self._running = 2 # ask main loop to finish
 		print "ThreadedAppServer is shutting down..."
 		sys.stdout.flush()
 		self.awakeSelect() # unblock select call in mainloop()
 		for i in range(30): # wait at most 3 seconds for shutdown
-			if self.running < 2:
+			if self._running < 2:
 				break
 			time.sleep(0.1)
 		# Close all sockets now:
@@ -485,25 +494,25 @@ class ThreadedAppServer(AppServer):
 			sock.close()
 		# Remove the text files with the server addresses:
 		for handler in self._socketHandlers.values():
-			adr_file = self.addressFileName(handler)
-			if os.path.exists(adr_file):
+			adrFile = self.addressFileName(handler)
+			if os.path.exists(adrFile):
 				try:
-					os.unlink(adr_file)
+					os.unlink(adrFile)
 				except OSError:
-					print "Warning: Could not remove", adr_file
+					print "Warning: Could not remove", adrFile
 		# Tell all threads to end:
 		for i in range(self._threadCount):
 			self._requestQueue.put(None)
 		for i in self._threadPool:
 			try:
 				i.join()
-			except:
+			except Exception:
 				pass
 		# Call super's shutdown:
 		AppServer.shutDown(self)
 		sys.stdout.flush()
 		sys.stderr.flush()
-		self.running = 0
+		self._running = 0
 
 	def awakeSelect(self):
 		"""Awake the select() call.
@@ -521,7 +530,7 @@ class ThreadedAppServer(AppServer):
 			try:
 				sock.connect((host, port))
 				sock.close()
-			except:
+			except Exception:
 				pass
 
 
@@ -560,7 +569,7 @@ class ThreadedAppServer(AppServer):
 
 	def addressFileName(self, handlerClass):
 		"""Get the name of the text file with the server address."""
-		return ('%s.text' % handlerClass.protocolName)
+		return '%s.address' % handlerClass.protocolName
 
 
 class Handler:
@@ -684,14 +693,14 @@ class MonitorHandler(Handler):
 	and returns a value indicating the status of the server.
 
 	The protocol passes a marshalled dict, much like the Adapter
-	interface, which looks like ``{'format': 'XXX'}``, where XXX
+	interface, which looks like ``{'format': 'CMD'}``, where CMD
 	is a command (``STATUS`` or ``QUIT``). Responds with a simple
 	string, either the number of requests we've received (for
 	``STATUS``) or ``OK`` for ``QUIT`` (which also stops the server).
 
 	"""
 	# @@ 2003-03 ib: we should have a RESTART command, and
-	# perhaps better status indicators (# of threads, etc).
+	# perhaps better status indicators (number of threads, etc).
 
 	protocolName = 'monitor'
 	settingPrefix = 'Monitor'
@@ -701,11 +710,10 @@ class MonitorHandler(Handler):
 		startTime = time.time()
 		if verbose:
 			print "BEGIN REQUEST"
-			print time.asctime(time.localtime(startTime))
+			print asclocaltime(startTime)
 		conn = self._sock
 		if verbose:
 			print "receiving request from", conn
-		BUFSIZE = 8*1024
 		requestDict = self.receiveDict()
 		if requestDict['format'] == "STATUS":
 			conn.send(str(self._server._requestID))
@@ -715,27 +723,32 @@ class MonitorHandler(Handler):
 			self._server.shutDown()
 
 
-from WebKit.ASStreamOut import ASStreamOut
-class TASASStreamOut(ASStreamOut):
+silentErrnos = [] # silently ignore these errors:
+for e in 'EPIPE', 'ECONNABORTED', 'ECONNRESET':
+	try:
+		silentErrnos.append(getattr(errno, e))
+	except AttributeError:
+		pass
+
+
+class TASStreamOut(ASStreamOut):
 	"""Response stream for ThreadedAppServer.
 
-	The `TASASStreamOut` class streams to a given socket, so that when `flush`
+	The `TASStreamOut` class streams to a given socket, so that when `flush`
 	is called and the buffer is ready to be written, it sends the data from the
 	buffer out on the socket. This is the response stream used for requests
 	generated by ThreadedAppServer.
 
-	TAS stands for ThreadedAppServer (AS for AppServer... a little redundant).
-
 	"""
 
-	def __init__(self, sock):
+	def __init__(self, sock, autoCommit=False, bufferSize=8192):
 		"""Create stream.
 
 		We get an extra `sock` argument, which is the socket which we'll
 		stream output to (if we're streaming).
 
 		"""
-		ASStreamOut.__init__(self)
+		ASStreamOut.__init__(self, autoCommit, bufferSize)
 		self._socket = sock
 
 	def flush(self):
@@ -747,22 +760,19 @@ class TASASStreamOut(ASStreamOut):
 
 		"""
 		result = ASStreamOut.flush(self)
-		if result: # a True return value means we can send
+		if result: # a true return value means we can send
 			reslen = len(self._buffer)
 			sent = 0
+			bufferSize = self._bufferSize
 			while sent < reslen:
 				try:
 					sent += self._socket.send(
-						self._buffer[sent:sent+8192])
+						self._buffer[sent:sent+bufferSize])
 				except socket.error, e:
-					if e[0] == errno.EPIPE: # broken pipe
-						pass
-					elif hasattr(errno, 'ECONNRESET') \
-							and e[0] == errno.ECONNRESET:
-						pass
-					else:
-						print "StreamOut Error: ", e
-					break
+					if debug or e[0] not in silentErrnos:
+						print "StreamOut Error:", e
+					self._closed = True
+					raise ConnectionAbortedError
 			self.pop(sent)
 
 
@@ -782,13 +792,13 @@ class AdapterHandler(Handler):
 	to handle that data.
 
 	"""
-	protocolName = 'address'
+	protocolName = 'adapter'
 	settingPrefix = 'Adapter'
 
 	def handleRequest(self):
 		"""Handle request.
 
-		Creates the request dictionary, and creates a `TASASStreamOut` object
+		Creates the request dictionary, and creates a `TASStreamOut` object
 		for the response, then calls `Application.dispatchRawRequest`, which
 		does the rest of the work (here we just clean up after).
 
@@ -801,27 +811,32 @@ class AdapterHandler(Handler):
 			return
 
 		if verbose:
-			requestURI = requestDict.has_key('environ') \
-				and Funcs.requestURI(requestDict['environ']) or '-'
-			sys.stdout.write('%5i  %s  %s\n' % (self._requestID,
-				timestamp()['pretty'], requestURI))
+			uri = requestDict.has_key('environ') \
+				and requestURI(requestDict['environ']) or '-'
+			sys.stdout.write('%5i  %s  %s\n'
+				% (self._requestID, timestamp()['pretty'], uri))
 
 		requestDict['input'] = self.makeInput()
 		requestDict['requestID'] = self._requestID
-		streamOut = TASASStreamOut(self._sock)
+
+		streamOut = TASStreamOut(self._sock, bufferSize=self._server._responseBufferSize)
 		transaction = self._server._app.dispatchRawRequest(requestDict, streamOut)
-		streamOut.close()
+		try:
+			streamOut.close()
+			aborted = False
+		except ConnectionAbortedError:
+			aborted = True
 
 		try:
 			self._sock.shutdown(1)
 			self._sock.close()
-		except:
+		except Exception:
 			pass
 
 		if verbose:
 			duration = ('%0.2f secs' % (time.time() - self._startTime)).ljust(19)
-			sys.stdout.write('%5i  %s  %s\n\n' % (self._requestID,
-				duration, requestURI))
+			sys.stdout.write('%5i  %s  %s\n\n' % (self._requestID, duration,
+				aborted and '*connection aborted*' or uri))
 
 		transaction._application = None
 		transaction.die()
@@ -829,7 +844,8 @@ class AdapterHandler(Handler):
 
 	def makeInput(self):
 		"""Create a file-like object from the socket."""
-		return self._sock.makefile("rb", 8012)
+		return self._sock.makefile("rb", self._server._requestBufferSize)
+
 
 # Determines whether the main look should run in another thread.
 # On Win NT/2K/XP, we run the mainloop in a different thread because
@@ -842,18 +858,21 @@ def runMainLoopInThread():
 # Set to False in DebugAppServer so Python debuggers can trap exceptions:
 doesRunHandleExceptions = True
 
+
 class RestartAppServerError(Exception):
 	"""Raised by DebugAppServer when needed."""
 	pass
 
-os_chdir = os.chdir
+
+_chdir = os.chdir
 
 def chdir(path, force=False):
 	"""Execute os.chdir() with safety provision."""
 	assert force, \
 		"You cannot reliably use os.chdir() in a threaded environment.\n" \
 		+ 16*" " + "Set force=True if you want to do it anway (using a lock)."
-	os_chdir(path)
+	_chdir(path)
+
 
 ## Script usage ##
 
@@ -892,11 +911,11 @@ def run(workDir=None):
 						target=_windowsmainloop)
 					t.start()
 					try:
-						while server.running > 1:
+						while server._running > 1:
 							try:
 								time.sleep(1) # wait for interrupt
-							except:
-								if server.running < 3:
+							except Exception:
+								if server._running < 3:
 									raise # shutdown
 					finally:
 						t.join()
@@ -925,23 +944,26 @@ def run(workDir=None):
 					exitStatus = 0
 				else:
 					if doesRunHandleExceptions:
-						import traceback
-						print
-						traceback.print_exc(file=sys.stderr)
-						print "Exiting AppServer due to above exception."
+						if not server and isinstance(e, ProcessRunning):
+							print "Error:", str(e)
+						else:
+							print
+							traceback.print_exc()
+							print
+							print "Exiting AppServer due to above exception."
 						exitStatus = 1
 					else:
 						raise
 		finally:
 			sys.stdout.flush()
 			sys.stderr.flush()
-			if server and server.running:
+			if server and server._running:
 				server.initiateShutdown()
 				server._closeThread.join()
 			AppServerModule.globalAppServer = None
 	sys.stdout.flush()
 	sys.stderr.flush()
-	os.chdir = os_chdir # allow use of os.chdir() again
+	os.chdir = _chdir # allow use of os.chdir() again
 	return exitStatus
 
 # Signal handlers
@@ -950,10 +972,10 @@ def shutDown(signum, frame):
 	"""Signal handler for shutting down the server."""
 	print
 	print "App server has been signaled to shutdown."
-	if server and server.running > 2:
-		print "Shutting down at", time.asctime(time.localtime(time.time()))
+	if server and server._running > 2:
+		print "Shutting down at", asclocaltime()
 		sys.stdout.flush()
-		server.running = 2
+		server._running = 2
 		if signum == SIGINT:
 			raise KeyboardInterrupt
 		elif signum == SIGHUP:
@@ -973,7 +995,7 @@ try:
 		print
 		print "App server has been signaled to attempt a thread dump."
 		print
-		print "Thread stack frame dump at", time.asctime(time.localtime(time.time()))
+		print "Thread stack frame dump at", asclocaltime()
 		sys.stdout.flush()
 		frames = threadframe.dict()
 		items = frames.items()
@@ -981,9 +1003,9 @@ try:
 		print
 		print "-" * 79
 		print
-		for thread_id, frame in items:
+		for threadId, frame in items:
 			print "Thread ID: %d (reference count = %d)" % (
-				thread_id, sys.getrefcount(frame))
+				threadId, sys.getrefcount(frame))
 			print ''.join(traceback.format_list(traceback.extract_stack(frame)))
 		items.sort()
 		print "-" * 79
@@ -1054,8 +1076,7 @@ def main(args):
 			value = i[match.end():]
 			Configurable.addCommandLineSetting(name, value)
 		elif i == "stop":
-			import AppServer
-			function = AppServer.stop
+			function = AppServerModule.stop
 		elif i == "daemon":
 			daemon = True
 		elif i == "start":
