@@ -12,25 +12,26 @@ up by `Application` (accessible through `Application.rootURLParser`).
 
 """
 
-
 import re, os, sys
+
+from HTTPExceptions import HTTPNotFound, HTTPMovedPermanently
 from MiscUtils.ParamFactory import ParamFactory
-from WebKit.HTTPExceptions import *
 from WebUtils.Funcs import urlDecode
 
 debug = 0
 
-try: # backward compatibility for Python < 2.1
+try: # for Python < 2.1
 	from warnings import warn
 except ImportError:
-	def warn(s, **a):
-		print s
+	def warn(message, **args):
+		print message
 
 # Legal characters for use in a module name -- used when turning
 # an entire path into a module name.
-moduleNameRE = re.compile('[^a-zA-Z_]')
+_moduleNameRE = re.compile('[^a-zA-Z_]')
 
 _globalApplication = None
+
 def application():
 	"""Returns the global Application."""
 	return _globalApplication
@@ -102,7 +103,7 @@ class ContextParser(URLParser):
 		# Need to set this here because we need for initialization, during
 		# which AppServer.globalAppServer.application() doesn't yet exist:
 		self._app = app
-		self._imp= app._imp
+		self._imp = app._imp
 		# self._context will be a dictionary of context
 		# names and context directories.  It is set by
 		# `addContext`.
@@ -153,7 +154,6 @@ class ContextParser(URLParser):
 		"""
 		contexts = self._contexts
 		contextDirs = {}
-		app = self._app
 		# make a list of existing context paths
 		for name, path in contexts.items():
 			if name != 'default':
@@ -206,34 +206,38 @@ class ContextParser(URLParser):
 					if os.path.exists(dir):
 						f = os.path.join(dir, '__init__.py')
 						if not os.path.exists(f):
-							print 'Creating __init__.py file for context:', name
+							print 'Creating ' \
+								'__init__.py file for context:', name
 							try:
-								file(f, 'w').write('# Auto-generated by WebKit' + os.linesep)
-							except:
-								print 'Error: __init__.py file could not be created.'
+								open(f, 'w').write(
+									'# Auto-generated by WebKit' + os.linesep)
+							except Exception:
+								print 'Error: ' \
+									'__init__.py file could not be created.'
 							else:
-								res = self._imp.find_module(packageName, [localDir])
+								res = self._imp.find_module(packageName,
+									[localDir])
 								if res:
 									e = None
 					if e:
 						raise
 				mod = self._imp.load_module(name, *res)
-		except ImportError, e:
+		except (ImportError, TypeError), e:
+			# TypeError can be raised by imp.load_module()
+			# when the context path does not exist
 			pass
-		except TypeError, e: # TypeError can be raised by imp.load_module()
-			pass # when context path does not exist
 		if e:
 			print 'Error loading context: %s: %s: dir=%s' % (name, e, dir)
 			return
 
 		if hasattr(mod, 'contextInitialize'):
 			# @@ gat 2003-07-23: switched back to old method
-			# @@ of passing application as first parameter
-			# @@ to contextInitialize for backward compatibility
+			# of passing application as first parameter
+			# to contextInitialize for backward compatibility
 			result = mod.contextInitialize(application(),
 				os.path.normpath(os.path.join(os.getcwd(), dir)))
 			# @@: funny hack...?
-			if result != None and result.has_key('ContentLocation'):
+			if result is not None and result.has_key('ContentLocation'):
 				dir = result['ContentLocation']
 
 		print 'Loading context: %s at %s' % (name, dir)
@@ -264,20 +268,21 @@ class ContextParser(URLParser):
 		through `Request.serverSidePath` and `Request.contextName`).
 
 		"""
-		# This is a hack...
-		# this should probably go in the Transaction class:
+		# This is a hack... should probably go in the Transaction class:
 		trans._fileParserInitSeen = {}
+		# If there is no path, redirect to the root path:
 		req = trans.request()
 		if not requestPath:
-			p = '/'
+			p = req.servletPath() + '/'
 			q = req.queryString()
 			if q:
 				p += "?" + q
 			raise HTTPMovedPermanently(location=p)
+		# Determine the context name:
 		if req._absolutepath:
 			contextName = self._defaultContext
 		else:
-			context = filter(lambda x: x, requestPath.split('/'))
+			context = filter(None, requestPath.split('/'))
 			if requestPath.endswith('/'):
 				context.append('')
 			parts = []
@@ -558,26 +563,22 @@ class _FileParser(URLParser):
 
 	def initModule(self):
 		"""Get the __init__ module object for this FileParser's directory."""
+		path = self._path
+		# if this directory is a context, return the context package
+		for context, dir in self._app.contexts().items():
+			if dir == path:
+				# avoid reloading of the context package
+				return sys.modules[context]
+		name = 'WebKit_Cache_' + _moduleNameRE.sub('_', path)
 		try:
-			result = self._imp.find_module('__init__', [self._path])
-			if result is None:
-				return None
-			file, initPath, desc = result
-			fullName = moduleNameRE.sub('_', os.path.join(self._path, '__init__'))
-			if len(fullName) > 100:
-				fullName = fullName[:-50]
-			module = None
-			try:
-				module = self._imp.load_module(fullName, file, initPath, desc)
-			except:
-				pass
-			file.close()
+			file, path, desc = self._imp.find_module('__init__', [path])
+			module = self._imp.load_module(name, file, path, desc)
 			return module
-		except ImportError:
-			return None
+		except (ImportError, TypeError):
+			pass
 
 	def parseInit(self, trans, requestPath):
-		"""Partse the __init_ file.
+		"""Parse the __init_ file.
 
 		Returns the resulting servlet, or None if no __init__ hooks were found.
 
@@ -747,14 +748,14 @@ class URLParameterParser(URLParser):
 	## Init ##
 
 	def __init__(self, fileParser=None):
-		self.fileParser = fileParser
+		self._fileParser = fileParser
 
 
 	## Parsing ##
 
 	def parse(self, trans, requestPath):
 		"""Delegates to `parseHook`."""
-		return self.parseHook(trans, requestPath, self.fileParser)
+		return self.parseHook(trans, requestPath, self._fileParser)
 
 	def parseHook(self, trans, requestPath, hook):
 		"""Munges the path.
@@ -826,7 +827,7 @@ class ServletFactoryManagerClass:
 
 	def factoryForFile(self, path):
 		"""Get a factory for a filename."""
-		name, ext = os.path.splitext(path)
+		ext = os.path.splitext(path)[1]
 		if self._factoryExtensions.has_key(ext):
 			return self._factoryExtensions[ext]
 		if self._factoryExtensions.has_key('.*'):
@@ -873,17 +874,16 @@ def initApp(app):
 def initParser(app):
 	"""Initialize the FileParser Class."""
 	cls = _FileParser
+	cls._app = app
 	cls._imp = app._imp
+	cls._contexts = app.contexts
 	cls._filesToHideRegexes = []
 	cls._filesToServeRegexes = []
 	from fnmatch import translate as fnTranslate
-	import re
 	for pattern in app.setting('FilesToHide'):
-		cls._filesToHideRegexes.append(
-			re.compile(fnTranslate(pattern)))
+		cls._filesToHideRegexes.append(re.compile(fnTranslate(pattern)))
 	for pattern in app.setting('FilesToServe'):
-		cls._filesToServeRegexes.append(
-			re.compile(fnTranslate(pattern)))
+		cls._filesToServeRegexes.append(re.compile(fnTranslate(pattern)))
 	cls._toIgnore = app.setting('ExtensionsToIgnore')
 	cls._toServe = app.setting('ExtensionsToServe')
 	cls._useCascading = app.setting('UseCascadingExtensions')
