@@ -1,4 +1,5 @@
-"""
+"""HTMLTag.py
+
 HTMLTag defines a class of the same name that represents HTML content. An
 additional HTMLReader class kicks off the process of reading an HTML file into a
 set of tags:
@@ -49,6 +50,7 @@ is broken for empty tags).
 
 * Determined what HTML tags are empty by scanning O'Reilly's HTML Pocket
 Reference.
+
 """
 
 # - report line numbers for errors
@@ -61,7 +63,7 @@ pyVer = getattr(sys, 'version_info', None)
 if pyVer is None or pyVer[0] < 2:
 	raise Exception, 'HTMLTag requires Python 2.0 or greater.'
 
-try: # backward compatibility for Python < 2.3
+try: # for Python < 2.3
 	True, False
 except NameError:
 	True, False = 1, 0
@@ -71,8 +73,7 @@ runFast = True
 	# changing this has no effect once the module is imported (unless you reload())
 
 from sgmllib import SGMLParser
-from MiscUtils import NoDefault
-import types
+from MiscUtils import NoDefault, AbstractError
 from types import StringType
 
 
@@ -89,7 +90,8 @@ class HTMLTagProcessingInstructionError(HTMLTagError): pass
 class HTMLTagIncompleteError(HTMLTagError): pass
 
 
-DefaultEmptyTags = 'area basefont base bgsound br col colgroup frame hr img input isindex link meta spacer wbr'.split()
+DefaultEmptyTags = 'area basefont base bgsound br col colgroup frame hr' \
+	' img input isindex link meta spacer wbr'.split()
 
 
 class HTMLTag:
@@ -145,6 +147,7 @@ class HTMLTag:
 	  have spaces as in "  quoted  ". But that's speculation.
 	"""
 
+
 	## Init and reading ##
 
 	def __init__(self, name, lineNumber=None):
@@ -169,7 +172,7 @@ class HTMLTag:
 		Adds a child to the receiver. The child will be another tag or a string
 		(CDATA).
 		"""
-		assert isinstance(child, HTMLTag) or type(child) is types.StringType, \
+		assert isinstance(child, HTMLTag) or type(child) is StringType, \
 			'Invalid child: %r' % child
 		self._children.append(child)
 		if isinstance(child, HTMLTag):
@@ -298,6 +301,7 @@ class HTMLTag:
 		self._closedBy = name
 		self._closedAt = lineNumber
 
+
 	## Self utility ##
 
 	def _tagWithMatchingAttr(self, name, value):
@@ -388,6 +392,7 @@ class HTMLReader(SGMLParser):
 	  validation.
 	"""
 
+
 	## Init ##
 
 	def __init__(self, emptyTags=None, extraEmptyTags=None, fakeRootTagIfNeeded=True):
@@ -543,7 +548,7 @@ class HTMLReader(SGMLParser):
 	def handle_data(self, data):
 		if self._finished:
 			return
-		assert type(data) is types.StringType
+		assert type(data) is StringType
 		if self._ignoreWS:
 			data = data.strip()
 			if not data:
@@ -602,7 +607,7 @@ class HTMLReader(SGMLParser):
 		openingTag = self._tagStack.pop()
 		if self._printsStack:
 			print 'END   %s: %r' % (name.ljust(6), self._tagStack)
-		if openingTag.name()!=name:
+		if openingTag.name() != name:
 			raise HTMLTagUnbalancedError('line %i: opening is %r, but closing is <%s>.' % \
 				(self._lineNumber, openingTag, name), line=self._lineNumber, opening=openingTag.name(), closing=name, tagStack=self._tagStack)
 		else:
